#*****************************************************************************
#* File:  ADCASM.S
#* Version: 1.00,   15/10/1999
#* Author:  Philippe TECHER,  Virtual Micro Degin (ptecher@vmdesign.com) 
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - ....
#-----------------------------------------------------------------------------

        .name           "adcasm.s"
        .text
        .include        "555regsdef.s"

# Exported function

        .xdef          	ADCInit
        .xdef           POTSet
        .xdef          	ADCRead
        .xdef           DACWrite

#*****************************************************************************
# Syntax
# A_NameHi : Refers MSB 16bit Address  for "name" Module 
# A_NameLo : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
#-----------------------------------------------------------------------------

MaxE2POT	 = 100     # E2POT have 100 possible position	

A_QADCBase       = 0x0030  # base to access QADC Module   
Ofs_ADC0         = 0x0000  # Offset for module n0
Ofs_ADC1         = 0x0400  # Offset for module n1

A_QADC64MCR      = 0x4800  # Module Configuration register
A_QADC64TEST     = 0x4802  # Test Register
A_QADC64INT      = 0x4804  # Interrupt register
A_PORTQA         = 0x4806  # Port A Data
A_PORTQB         = 0x4807  # Port B Data
A_DDRQA          = 0x4808  # Port A Data Direction Register
A_QACR0          = 0x480A  # Control Register 0
A_QACR1          = 0x480C  # Control Register 1
A_QACR2          = 0x480E  # Control Register 2
A_QASR0          = 0x4810  # Status Register 0
A_QASR1          = 0x4812  # Status Register 1

A_CCW            = 0x4A00  # Conversion Command Word
A_ADCUDATA       = 0x4A80  # Result Word Table, Right-Just, Unsigned Result Reg

#-----------------------------------------------------------------------------

A_MIOSBase       = 0x0030
A_MPIOSMDR       = 0x6100 # MPIOSM Data Register
A_MPIOSMDDR      = 0x6102 # Data Direction Register (MPIOSMDDR)

#-----------------------------------------------------------------------------

V_DDRQA_0        = 0x7F   # All PA0..PA6 as Ouput, PA7 as Input
V_DDRQA_1        = 0x0F   # PA0..PA3 as outpout, PA4..PA7 as input
V_PORTQA_0       = 0x01   # LCD Port all set to "0", (bit0=INC=1)
V_PORTQA_1       = 0x0F   # Chip select for XICOR Pot, all to "1"

#*****************************************************************************
# prototype:   int ADCInit (void)#
#-----------------------------------------------------------------------------
# Initialize ADC channel
#-----------------------------------------------------------------------------

ADCInit:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0 ,0x1C(r1)       # [SP + 0x1C] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase           # r31 = Pointer for QADC Module (0x0030 0000)

        li      r3,V_DDRQA_0                 # Set other PORT to correct values.
        stb     r3,A_DDRQA+Ofs_ADC0(r31)     # @0x304808 <- 0x7F Mod 0 Data Direction Reg       
        li      r3,V_DDRQA_1      
        stb     r3,A_DDRQA+Ofs_ADC1(r31)     # @0x304C08 <- 0x0f Mod 1 Data Direction Reg             
        li      r3,V_PORTQA_0
        stb     r3,A_PORTQA+Ofs_ADC0(r31)    # @0x304806 <- 0x01 Mod 0 Port A Data                     
        li      r3,V_PORTQA_1      
        stb     r3,A_PORTQA+Ofs_ADC1(r31)    # @0x304C06 <- 0x0F Mod 1 Port A Data                            

        li      r3,0               		   # see 13.12.1, no special setting
        sth     r3,A_QADC64MCR+Ofs_ADC0(r31) # @0x304800 <- 0x0 Mod 0 Module Configuration Reg                            
        sth     r3,A_QADC64MCR+Ofs_ADC1(r31) # @0x304800 <- 0x0 Mod 1 Module Configuration Reg                            

        li      r3,0x00B7                    # PSH=11, PSA=0, PSL=7 (see: 13.12.6) 
        sth     r3,A_QACR0+Ofs_ADC0(r31)	   # @0x30480A <- 0x00B7 Mod 0 Control Reg 0                           
        sth     r3,A_QACR0+Ofs_ADC1(r31)     # @0x304C0A <- 0x00B7 Mod 1 Control Reg 0     
                      
        li      r3,0x1100                    # Op=b10001: Software triggered continous scan mode 
        sth     r3,A_QACR1+Ofs_ADC0(r31)     # @0x30480C <- 0x00B7 Mod 0 Control Reg 1                           
        sth     r3,A_QACR1+Ofs_ADC1(r31)     # @0x304C0C <- 0x00B7 Mod 1 Control Reg 1                           

        li      r3,0x0040                    # QUEUE 2 disabled
        sth     r3,A_QACR2+Ofs_ADC0(r31)     # @0x30480E <- 0x0040 Mod 0 Control Reg 2                           
        sth     r3,A_QACR2+Ofs_ADC1(r31)     # @0x304C0E <- 0x0040 Mod 1 Control Reg 2                            

                                         # ***** @0x304A00 Mod 0 Conversion Command Word *****
        li      r3,0x0000                # default value for QUEUE
        ori     r0,r3,0x0000             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0(r31)   # Channel A_PQB0
        ori     r0,r3,0x0001             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+2(r31) # Channel A_PQB1
        ori     r0,r3,0x0002             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+4(r31) # Channel A_PQB2
        ori     r0,r3,0x0003             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+6(r31) # Channel A_PQB3
        ori     r0,r3,0x0030             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+8(r31) # Channel A_PQB4
        ori     r0,r3,0x0031             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+10(r31)# Channel A_PQB5
        ori     r0,r3,0x0032             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+12(r31)# Channel A_PQB6
        ori     r0,r3,0x0033             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC0+14(r31)# Channel A_PQB7
        ori     r0,r3,0x003E             # Measure (VRH-VRL / 2)
        sth     r0,A_CCW+Ofs_ADC0+16(r31)# reserved
        ori     r0,r3,0x003F             # END of QUEUE
        sth     r0,A_CCW+Ofs_ADC0+18(r31)


                                         # ***** @0x304E00 Mod 1 Conversion Command Word *****
        ori     r0,r3,0x0000             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1(r31)   # Channel A_PQB0
        ori     r0,r3,0x0001             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+2(r31) # Channel A_PQB1
        ori     r0,r3,0x0002             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+4(r31) # Channel A_PQB2
        ori     r0,r3,0x0003             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+6(r31) # Channel A_PQB3
        ori     r0,r3,0x0030             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+8(r31) # Channel A_PQB4
        ori     r0,r3,0x0031             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+10(r31)# Channel A_PQB5
        ori     r0,r3,0x0032             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+12(r31)# Channel A_PQB6
        ori     r0,r3,0x0033             # "OR" default value to channel value into r0
        sth     r0,A_CCW+Ofs_ADC1+14(r31)# Channel A_PQB7
        ori     r0,r3,0x003E             # Measure (VRH-VRL / 2)
        sth     r0,A_CCW+Ofs_ADC1+16(r31)# reserved
        ori     r0,r3,0x003F             # END of QUEUE
        sth     r0,A_CCW+Ofs_ADC1+18(r31)

	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:   int ADCRead (int Channel, int *Value)
#-----------------------------------------------------------------------------
# Read an ADC channel (Channel = 0..15)
#-----------------------------------------------------------------------------
# Ch-00 : HE10.4                Ch-01 : HE10.6
# Ch-02 : HE10.8                Ch-03 : HE10.9
# Ch-04 : HE10.7                Ch-05 : HE10.5
# Ch-06 : HE10.3                Ch-07 : HE10.1
# Ch-08 : VAxe-1                Ch-09 : VAxe-2
# Ch-10 : VAxe-3                Ch-11 : VAxe-4
# Ch-12 : Temperature-1         Ch-13 : Temperature-2
# Ch-14 : Temperature-3         Ch-15 : Temperature-4
#-----------------------------------------------------------------------------

ADCRead:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0 ,0x1C(r1)       # [SP + 0x1C] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase # r31 = Pointer for QADC Module (0x0030 0000)
        andi.   r3,r3,0x0F
        andi.   r0,r3,0x08
        beq     ADCRead_00
        addi    r31,r31,Ofs_ADC1   # Channel 8..15 are located on module "B"
ADCRead_00:
        rlwinm  r3,r3,1,28,30      # SHIFT Left and keep only bit 28..30
        add     r31,r31,r3         # compute exact address
        lhz     r0,A_ADCUDATA(r31) # r0 = Value to read
        stw     r0,0(r4)           # Store value

        li      r3,0               # return 0
	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:   int PotSet (int PotCh, int Value)
#-----------------------------------------------------------------------------
# Increment (Value>=0) or Decrement (Value<0) the XICOR potentiometer PotCh
# with "ABS(Value)" pulse.
#-----------------------------------------------------------------------------

POTSet:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0 ,0x1C(r1)       # [SP + 0x1C] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase # r31 = Pointer for QADC Module (0x0030 0000)

        li      r0,V_DDRQA_0       # Set other PORT to correct values.
        stb     r0,A_DDRQA+Ofs_ADC0(r31)        
        li      r0,V_DDRQA_1      
        stb     r0,A_DDRQA+Ofs_ADC1(r31)        

        li      r0,V_PORTQA_0
        stb     r0,A_PORTQA+Ofs_ADC0(r31)        

        li      r0,V_PORTQA_1      
        stb     r0,A_PORTQA+Ofs_ADC1(r31)        
        eieio                      # to be sure the PORT is updated
	
                                   # --- SELECT The U/D pin FIRST
        li      r0,2               # Set U/D Pin to the correct value
        andi.   r3,r4,0x8000       # is VALUE < 0 ?
        beq     POTSet_00
        li      r0,0               # VALUE<0, U/D Pin = 0
        neg     r4,r4              # retrive a positive value
POTSet_00:
        stb     r0,A_PORTQA(r31)   # Store into good port
	eieio
                                   # --- SELECT The E2POT XICOR CHIP
        andi.   r3,r3,0x03         # keep only 2 bits, because we have only 4 channels
        li      r0,1
        rlwnm   r0,r0,r3,28,31     # SHIFT Left r3 time and keep only bit 28..31
        xori    r0,r0,0x0F         # Negate the result (XICOR CS is negated)
        stb     r0,A_PORTQA+Ofs_ADC1(r31) # Store into good port
        eieio                       # to be sure the PORT is updated

        cmpi	cr0,0,r4,0         # r4 = 0 ?
        beq     POTSet_03
        cmpi    cr0,0,r4,MaxE2POT  # r4 > MaxE2POT
        ble     POTSet_01
        li      r4,100             # Yes, keep r4 to 100, because Xicor POT have only
                                   # 100 positions available
POTSet_01:
        mtspr   9,r4               # CTR = R4 and will serve to count pulses
POTSet_02:
        li      r3,0x00
        or      r3,r0,r3
        stb     r3,A_PORTQA(r31)   # Store into good port
        eieio
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop

        li      r3,0x01
        or      r3,r0,r3
        stb     r3,A_PORTQA(r31)   # Store into good port
        eieio
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop

        bdnz    POTSet_02           # Loop until the correct number of pulse is done

POTSet_03:
        li      r3,0x01
        or      r3,r0,r3
        stb     r3,A_PORTQA(r31)   # Store into good port
        eieio
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop

        li      r0,0x0F             # DISABLE ALL Xicor CS
        stb     r0,A_PORTQA+Ofs_ADC1(r31) # Store into good port
        eieio

        li      r3,0               # return 0
	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	



#*****************************************************************************
# prototype:   int DACWrite (int Ch, int Value)
#-----------------------------------------------------------------------------
# Write VALUE to the DAC channel Ch, this procedure DO NOT modify MIOS I/O.
#-----------------------------------------------------------------------------

dac_SDI         =  0x01
dac_HCK         =  0x02
dac_LDB         =  0x04
dac_MASK	=  0xFF8

WaitDAC:
        mfspr   r28,9
        li      r0,0x1    # Wait about 100 uS
        mtspr   9,r0
wLoop:
        nop
        nop
        nop
        eieio
        bdnz   wLoop
        mtspr   9,r28
        blr

DACWrite:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x08(r1)	   # [SP + 0x08] = r31
	stw	r30,0x0C(r1)
	stw	r29,0x10(r1)
	stw	r28,0x14(r1)
	stw	r0 ,0x1C(r1)

        li      r31,0  
        addis   r31,r31,A_MIOSBase # r31 = Pointer for MIOS Module (0x0030 0000)

        li      r0,0x0FFF          # b0..b3 Input, b4..b31 Output
        sth     r0,A_MPIOSMDDR(r31)# write DDR register

        lhz     r30,A_MPIOSMDR(r31) # Read PORT Value
        andi.   r30,r30,dac_MASK    # keep only b29, b30, b31

        ori     r0,r30,dac_HCK | dac_SDI  # HCK =0, SDI=0

  xori r0,r0,0x07
        sth     r0,A_MPIOSMDR(r31) # write "101" to DAC Serial PORT 
                                   # r30 = default value (hck=0, ldb=1)
        eieio                      # update I/O
	bl      WaitDAC

        rlwinm  r3,r3,14,16,15     # SHIFT Left 14 time and keep only bit 16,15
        andi.   r4,r4,0x0FFF       # keep only 12 bit in r4
        or      r3,r3,r4           # finally r3 = the value to serialize

        li      r0,16              # r0 = 16
        mtspr   9,r0               # CTR = 16

	b      DAC_00

DAC_Loop:
        ori     r29,r29,dac_HCK    # default value into r29
  xori r29,r29,0x07
        sth     r29,A_MPIOSMDR(r31)# write "10D" to DAC Serial PORT 
  xori r29,r29,0x07
        eieio		           # HCK = 0		
	bl      WaitDAC
        rlwinm  r3,r3,1,16,31     # SHIFT Left 1 time and keep only bit 16..31

DAC_00:
        ori     r29,r30,dac_HCK    # default value into r29
        andi.   r4,r3,0x8000       # keep r4 bit 16
        bne     DAC_01
        ori     r29,r29,dac_SDI
DAC_01:
  xori r29,r29,0x07
        sth     r29,A_MPIOSMDR(r31)# write "10D" to DAC Serial PORT 
  xori r29,r29,0x07
        eieio		           # HCK = 0		
	bl      WaitDAC
 
        andi.   r29,r29,(dac_SDI | dac_MASK)

  xori r29,r29,0x07
        sth     r29,A_MPIOSMDR(r31)# write "11D" to DAC Serial PORT 
  xori r29,r29,0x07
        eieio		           # HCK = 1
	bl      WaitDAC

        bdnz    DAC_Loop           # Loop until CTR = 0

        ori     r29,r29,dac_HCK    # default value into r29
  xori r29,r29,0x07
        sth     r29,A_MPIOSMDR(r31)# write "10D" to DAC Serial PORT 
  xori r29,r29,0x07
        eieio		           # HCK = 0		
	bl      WaitDAC

        ori     r29,r29,(dac_LDB | dac_HCK)
  xori r29,r29,0x07
        sth     r29,A_MPIOSMDR(r31)# write "00D" to DAC Serial PORT, LOAD the data 
  xori r29,r29,0x07
        eieio
	bl      WaitDAC

	andi.   r29,r29,(dac_HCK | dac_SDI | dac_MASK)
  xori r29,r29,0x07
        sth     r29,A_MPIOSMDR(r31)# write "10D" to DAC Serial PORT, LOAD the data 
  xori r29,r29,0x07
        eieio
	bl      WaitDAC

        li      r3,0               # return 0

	lwz	r31,0x08(r1)       # restore register and LR
	lwz	r30,0x0C(r1)
	lwz	r29,0x10(r1)
	lwz	r28,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#-----------------------------------------------------------------------------

