#*****************************************************************************
#* File:  CANASM.S
#* Version: 1.00,   25/11/1999
#* Author:  Philippe TECHER,  Virtual Micro Degin (ptecher@vmdesign.com) 
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - stw r0,A_RXGMSKHI(r31) instead of  stw r0,A_RXGMS(r31), r0=0x28
#-----------------------------------------------------------------------------

                 .name  "canasm.s"
                 .text
		 .include "555regsdef.s"

# Exported function

                 .xdef  CANOpen
                 .xdef  CANClose
                 .xdef  CANReadError
                 .xdef  CANSendMsg
                 .xdef  CANRecvMsg

#-----------------------------------------------------------------------------
#-------------- Switch to DATA Space
                 .data
                 

                 .text
#-------------- END Switch to DATA Space

#-----------------------------------------------------------------------------
# Here the structure used to send and receive message, only the pointer is
# passed to the function as parameter.
#  typedef struct
#    {
#      word id#        //  + 0x00
#      byte rtr#       //  + 0x02
#      byte dlen#      //  + 0x03
#      byte data[8]#   //  + 0x04
#    }TCanMsgStruct#
#
#*****************************************************************************
# Syntax
# A_Name   : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
#-----------------------------------------------------------------------------

# Err_nCAN         = 0x8001   # CAN Module selection out of range (nCAN!=0 or nCAN!=1)
Err_nCAN         = -0x7FFF   # CAN Module selection out of range (nCAN!=0 or nCAN!=1)
Err_NoMsg        = 0x0001
Err_NotFree      = 0x0002

A_TOUCANBASE     = 0x0030   # TouCAN Base, upper address (0x00300000)
BaseCAN_A        = 0x0000   # Offset to access CAN Module "A"
BaseCAN_B        = 0x0400   # Offset to access CAN Module "B"

A_TCNMCR         = 0x7080   # TouCAN Configuration Register (16.7.1)
A_TTRA           = 0x7082   # TouCAN Test Register (16.7.2)
A_CANICR         = 0x7084   # TouCAN Interrupt Register (16.7.3)
A_CANCTRL0       = 0x7086   # TouCAN Control Register 0 (16.7.4)
A_CANCTRL1       = 0x7087   # TouCAN Control Register 1 (16.7.5) - BYTE ACCESS - 
A_PRESDIV        = 0x7088   # TouCAN Control and Prescaler (16.7.6)
A_CANCTRL2       = 0x7089   # TouCAN Control Register 2 (16.7.7) - BYTE ACCESS -
A_TIMER          = 0x708A   # TouCAN Free running TIMER (16.7.8)
A_RXGMSK         = 0x7090
A_RXGMSKHI       = 0x7090   # TouCAN Receive Global Mask Registers HI (16.7.9)
A_RXGMSKLO       = 0x7092   # TouCAN Receive Global Mask Registers LO
A_RX14MASKHI     = 0x7094   # TouCAN Receive 14 Mask Registers HI (16.7.10)
A_RX14MASKLO     = 0x7096   # TouCAN Receive 14 Mask Registers LO
A_RX15MASKHI     = 0x7098   # TouCAN Receive 15 Mask Registers HI (16.7.11)
A_RX15MASKLO     = 0x709A   # TouCAN Receive 15 Mask Registers LO
A_ESTAT          = 0x70A0   # TouCAN Error and Status register (16.7.12)
A_IMASK          = 0x70A2   # TouCAN Interrupt Mask (16.7.13) 
A_IFLAG          = 0x70A4   # TouCAN Interrupt Flag (16.7.14) 
A_BUFFER0        = 0x7100
A_BUFFER1        = 0x7110
A_BUFFER2        = 0x7120
A_BUFFER3        = 0x7130
A_BUFFER4        = 0x7140
A_BUFFER5        = 0x7150
A_BUFFER6        = 0x7160
A_BUFFER7        = 0x7170
A_BUFFER8        = 0x7180
A_BUFFER9        = 0x7190
A_BUFFER10       = 0x71A0
A_BUFFER11       = 0x71B0
A_BUFFER12       = 0x71C0
A_BUFFER13       = 0x71D0
A_BUFFER14       = 0x71E0
A_BUFFER15       = 0x71F0


cd_TXNOTREADY    = 0b10000000   # Code for TouCAN buffer (see table 16-3, section 16.3.1.1)
cd_TXSEND        = 0b11000000
cd_TXREQUESTALW  = 0b10100000
cd_TXREQUEST     = 0b11100000

cb_RXNOTACTIVE   = 0b00000000   # see section 16.3.1.2, table 16-2
cb_RXEMPTY       = 0b00100000
cb_RXFULL        = 0b01100000
cb_RXOVERRUN     = 0b01100000
cb_RXBUSY        = 0b00100000   

#*****************************************************************************
#  Entry: r3 = nCAN number (0 or 1), 
#  Output: r31 point to TouCAN module for CAN A or B, Zero flag
#  is reset to return error in r3 register if nCAN is out of range
#-----------------------------------------------------------------------------

SetCANNum:
        li      r31,0  
        addis   r31,r31,A_TOUCANBASE # r31 = Pointer for TouCAN (0x0030 0000)
        cmpi	cr0,0,r3,0
        beq     SetCAN_00
        cmpi    cr0,0,r3,1
        bne     SetCAN_01
        addi    r31,r31,BaseCAN_B
        cmpi    cr0,0,r3,1         # set ZERO flag
SetCAN_00:
        blr
SetCAN_01:
        li      r3,Err_nCAN
        blr                        # Return: ZERO all is OK, Non ZERO, nCAN number is out of range


#*****************************************************************************
# prototype:   
#   int CANOpen (int nCan, int CanDiv)
#-----------------------------------------------------------------------------
# CanDiv determine CAN Clock f=ency as follow:
#         CanFreq = FSys / (CanDiv + 1) 
# For 40Mhz system clock:  250K -> 19, 500K -> 9, 1M -> 4
# WARNING: Presdiv MUST be < 256 !!!!!
# possible to set other lower rate using different CANCTRL0/1 setting
#-----------------------------------------------------------------------------
# For more detail see CAN Version 2.0, Section B.17, Timing configuration
# BitTIME = [ 1 + (PropSeg+1) + (PSeg1+1) + (PSeg2+1) ] x CanFreq
#  If PropSeg=2, PSeg1=1, PSeg2=1,    BitTIME = 8 x CanFreq  
#-----------------------------------------------------------------------------


CANOpen:
        stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r30,8(r1)          # [SP + 0x08] = r30
        stw	r31,0xC(r1)        # [SP + 0x0C] = r31
        stw	r0,0x14(r1)        # [SP + 0x14] = r0

        mr      r30,r3             # R30=R3
        bl      SetCANNum
        bne     CANOpen_End
                                   # see: 16.4.2 for exact init =ence
       
        li      r0,cd_TXNOTREADY   # TX-CAN code for buffer NOT ready
        sth     r0,A_BUFFER0(r31)  # Store first word into CAN Buffer 0
        sth     r0,A_BUFFER1(r31)  
        sth     r0,A_BUFFER2(r31)
        sth     r0,A_BUFFER3(r31)
        sth     r0,A_BUFFER4(r31)
        sth     r0,A_BUFFER5(r31)
        sth     r0,A_BUFFER6(r31)
        sth     r0,A_BUFFER7(r31)
        sth     r0,A_BUFFER8(r31)  # Store first word into CAN Buffer 8
        sth     r0,A_BUFFER9(r31)  
        sth     r0,A_BUFFER10(r31)
        sth     r0,A_BUFFER11(r31)
        sth     r0,A_BUFFER12(r31)
        sth     r0,A_BUFFER13(r31)
        sth     r0,A_BUFFER14(r31)
        sth     r0,A_BUFFER15(r31)

        li      r0,0x1200          # RESET CAN, no Wakeup interrupt, user & superuser register access
        sth     r0,A_TCNMCR(r31)   # Software CAN Reset =ence intiated, HALT Mode selected
        
        li      r0,0x0000          # set INT to level 0 (for CAN Module A )
        cmpi	cr0,0,r30,0        # R30 = CAN Module number (0 or 1)
        beq     CANOpen_01
        li      r0,0x0100          # set INT to level 1 ( for CAN Module B ) 
CANOpen_01:
        sth     r0,A_CANICR(r31)

        li      r0,0x0001
        sth     r0,A_IMASK(r31)    # Buffer 0 will activate an interrupt

#        li      r0,0x05            # Logic "1" is dominant bit for RX_0 and TX_0

        li      r0,0x00            # Logic "0" is dominant bit for RX_0 and TX_0
        stb     r0,A_CANCTRL0(r31) # Disable INT on CAN Error (16.7.4)

        li      r0,0x02            # PROPSEG=2,  no special setting
        stb     r0,A_CANCTRL1(r31)
        li      r0,0x09            # PSEG1=1, PSEG2=1, no special setting
        stb     r0,A_CANCTRL2(r31)
                                   # determine CAN Clock f=ency
        stb     r4,A_PRESDIV(r31)

        li      r0,0x28          
 #      stw     r0,A_RXGMSK(r31) # Store Mask registers NO ID is set, CAN will receive ALL frame 
        stw     r0,A_RXGMSKHI(r31) # Store Mask registers NO ID is set, CAN will receive ALL frame 

 #      stw     r0,A_RX14MASKHI(r31) # Not needed until now 
 #      stw     r0,A_RX15MASKLO(r31) # Not needed until now


        lhz     r0,A_ESTAT(r31)    # Read ESTAT and clear bits         

        li      r0,0x0000
        sth     r0,A_TCNMCR(r31)   # Software CAN Reset =ence intiated, NORMAL Mode selected

        li      r0,cd_TXNOTREADY   # TX-CAN code for buffer NOT ready
        sth     r0,A_BUFFER1(r31)  # Store first word into CAN Buffer 1
        li      r0,cb_RXEMPTY      # RX-CAN code to activate RX buffer 0 in RX Mode
        sth     r0,A_BUFFER0(r31)  
	li 	r0,0
        sth     r0,A_BUFFER0+2(r31)# Set MSG.ID  to 0000
        sth     r0,A_BUFFER0+4(r31)  

        li      r0,0x0002          # Set IFLAG.1 to prepare for first TX
        sth     r0,A_IFLAG(r31)      

        li	r3,0               # r3 = returned value = 0 = OK

CANOpen_End:
        lwz	r30,0x8(r1)
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return

#*****************************************************************************
# prototype:   
#   int CANClose (int nCan)
#-----------------------------------------------------------------------------


CANClose:
        stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r30,8(r1)          # [SP + 0x08] = r30
        stw	r31,0xC(r1)        # [SP + 0x0C] = r31
        stw	r0,0x14(r1)        # [SP + 0x14] = r0

        bl      SetCANNum
        bne     CANClose_End

        # li      r0,0x8000          # STOP CAN clock
        li      r0,-0x8000          # STOP CAN clock
        stw     r0,A_TCNMCR(r31)   # Store into TCNMCR Register

        li	r3,0               # r3 = returned value = 0 = OK

CANClose_End:
        lwz	r30,0x8(r1)
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return


#*****************************************************************************
# prototype:   
#   int CANSendMsg (byte nCAN, TCanMsgStruct *MsgPtr)#
#-----------------------------------------------------------------------------

CANSendMsg:
        stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r30,8(r1)          # [SP + 0x08] = r30
        stw	r31,0xC(r1)        # [SP + 0x0C] = r31
        stw	r0,0x14(r1)        # [SP + 0x14] = r0

        bl      SetCANNum
        bne     CANSend_End
                                   # here r4 point to MsgPtr, r31 point to TouCAN base address

        li      r3,Err_NotFree
        lhz     r0,A_IFLAG(r31)    # RO = IFlag
        andi.   r0,r0,0x0002       # is Buffer1 set IFLAG.1 ?
#        beq     CANSend_End        # no, no message present

#----------
        lhz     r0,A_IFLAG(r31)    # RO = IFlag
        andi.   r0,r0,0xFFFD       # Reset IFLAG.1 
        sth     r0,A_IFLAG(r31)      
        
#--- WARNING BUFFER1 is supposed to be free for transmission !
# See Section 16.4.3
#---- Disable transmit
        lbz     r0,0x03(r4)        # r1 = MsgPtr^.Len
        andi.   r0,r0,0x0F         # keep only 4 bit
        ori     r0,r0,cd_TXNOTREADY# Add TX-CAN code
        sth     r0,A_BUFFER1+0(r31)# Store first word into CAN Buffer 0
#---- Copy THE message
        lhz     r0,0x00(r4)        # r0 = MsgPtr^.ID
        rlwinm  r0,r0,8,16,23      # SHIFT Left 8 time and keep only bit 16..31
        sth     r0,A_BUFFER1+2(r31)# Store MSG ID

        lwz     r0,0x04(r4)        # r0 = MsgPtr^.(DATA[0]...DATA[3])
        stw     r0,A_BUFFER1+6(r31)
        lwz     r0,0x08(r4)        # r0 = MsgPtr^.(DATA[4]...DATA[7])
        stw     r0,A_BUFFER1+10(r31)
#---- Message is copyed, now enable transmit
        lhz     r0,0x02(r4)        # r1 = MsgPtr^.Len
        andi.   r0,r0,0x0F         # keep only 4 bit
        ori     r0,r0,cd_TXSEND    # Add TX-CAN code
        sth     r0,A_BUFFER1+0(r31)# Store first word into CAN Buffer 0

        li	r3,0               # r3 = returned value = 0 = OK
CANSend_End:
        lwz	r30,0x8(r1)
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return


#*****************************************************************************
# prototype:   
#   int CANRecvMsg (byte nCAN, TCanMsgStruct *MsgPtr)#
#-----------------------------------------------------------------------------
# return 0 if a message was trsnfered into MsgPtr
# else "Err_nCAN" or  "Err_NoMsg"
#-----------------------------------------------------------------------------

CANRecvMsg:
        stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r30,8(r1)          # [SP + 0x08] = r30
        stw	r31,0xC(r1)        # [SP + 0x0C] = r31
        stw	r0,0x14(r1)        # [SP + 0x14] = r0

        bl      SetCANNum
        bne     CANRecv_End
                                   # here r4 point to MsgPtr, r31 point to TouCAN base address

        li      r3,Err_NoMsg
        lhz     r0,A_IFLAG(r31)    # RO = IFlag
        andi.   r0,r0,0x0001       # is Buffer0 set IFLAG.0 ?
        beq     CANRecv_End        # no, no message present 
#----------
        lhz     r0,A_IFLAG(r31)    
        andi.   r0,r0,0xFFFE       # reset IFLAG.0        
        sth     r0,A_IFLAG(r31)    # IFlag = r0
#----------
        lhz     r0,A_BUFFER0+0(r31)# r0 = Status WORD, BUFFER 0 is now locked
        sth     r0,0x02(r4)        # store it, warning the structure must take care 
                                   # only of 4 low bit for dLen ! as it is a STATUS word that is copyed
        lhz     r0,A_BUFFER0+2(r31)# Read message ID
        srawi   r0,r0,8            # Shift 8 places to the right to keep only ID
        sth     r0,0x00(r4)        # store it 

        lwz     r0,A_BUFFER0+6(r31)# r0 = MsgPtr^.(DATA[0]...DATA[3])
        stw     r0,0x04(r4)        
        lwz     r0,A_BUFFER0+10(r31)# r0 = MsgPtr^.(DATA[4]...DATA[7])
        stw     r0,0x08(r4)        
#----------
# Reactivate BUFFER0, set Status word and ID to 0
        li      r0,cb_RXEMPTY      # RX-CAN code to activate RX buffer 0 in RX Mode
        sth     r0,A_BUFFER0(r31)  
	li 	r0,0
        sth     r0,A_BUFFER0+2(r31)# Set MSG.ID  to 0000
        sth     r0,A_BUFFER0+4(r31)  

        lhz     r0,A_TIMER(r31)    # Read free running timer to UNLOCK the Buffer0

        li	r3,0               # r3 = returned value = 0 = OK
CANRecv_End:
        lwz	r30,0x8(r1)
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return


