#*****************************************************************************
#* File:  INOASM.S
#* Version: 1.00,   20/10/1999
#* Author:  Philippe TECHER,  Virtual Micro Degin (ptecher@vmdesign.com) 
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - ....
#-----------------------------------------------------------------------------

        .name           "inoasm.s"
        .text
	.include        "555regsdef.s"

# Exported function

        .xdef          	ReadExIO
        .xdef           WriteExIO
        .xdef           ReadPWMIO
        .xdef           WritePWMIO
	.xdef		MPIOInit

#*****************************************************************************
# Syntax
# A_NameHi : Refers MSB 16bit Address  for "name" Module 
# A_NameLo : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
#-----------------------------------------------------------------------------

A_SIUBase        = 0x002F  # base to access SIU Module   
A_SIUOffset      = 0xC000  # Offset to add to obtain SIU base address


A_SGPIODT1       = 0x0024  # General Purpose IO Data Reg 1
A_SGPIODT2       = 0x0028  # General Purpose IO Data Reg 2
A_SGPIOCR        = 0x002C  # General Purpose IO Control Reg

#-----------------------------------------------------------------------------

A_MIOSBase       = 0x0030
A_MPIOSMDR       = 0x6100 # MPIOSM Data Register
A_MPIOSMDDR      = 0x6102 # Data Direction Register (MPIOSMDDR)

#-----------------------------------------------------------------------------

V_DDRQA_0        = 0x7F   # All PA0..PA6 as Ouput, PA7 as Input
V_DDRQA_1        = 0x0F   # PA0..PA3 as outpout, PA4..PA7 as input
V_PORTQA_0       = 0x01   # LCD Port all set to "0", (bit0=INC=1)
V_PORTQA_1       = 0x0F   # Chip select for XICOR Pot, all to "1"

#*****************************************************************************
# prototype:   int ReadExIO (void)
#-----------------------------------------------------------------------------

ReadExIO:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0,0x1C(r1)        # [SP + 0x1C] = r0

        li      r31,0
	addis   r31,r31,A_SIUBase
	ori	r31,r31,A_SIUOffset # USIU/Base 0x002FC000

        li      r3,0x00FF
        stw     r3,A_SGPIOCR(r31)   # USIU/SGPIOCR 0x002F002C

        lwz     r3,A_SGPIODT1(r31)  # USIU/SGPIOCR 0x002F0024
        srawi   r3,r3,8             # Shift 8 Place to the right
        andi.   r3,r3,0x00FF        # Keep only 8 bit


	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:   int WriteExIO (int Value)
#-----------------------------------------------------------------------------
# return the previous value as result
#-----------------------------------------------------------------------------

WriteExIO:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0,0x1C(r1)        # [SP + 0x1C] = r0

        li      r31,0
	addis   r31,r31,A_SIUBase
	ori	r31,r31,A_SIUOffset # USIU/Base 0x002FC000

        li      r30,0x00FF
        stw     r30,A_SGPIOCR(r31)  # USIU/SGPIOCR 0x002F002C

        lwz     r30,A_SGPIODT1(r31)# Read the port 
        andi.   r30,r30,0x00FF     # Keep only 8 bit
	andi.	r3,r3,0x00FF
        stw     r3,A_SGPIODT1(r31) # Write the port
        andi.   r3,r30,0x00FF      # Keep only 8 bit
                                   # r3 = Previous value

	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:   int ReadPWMIO (void)
#-----------------------------------------------------------------------------
# Return the port value (only 4 bit)
#-----------------------------------------------------------------------------

ReadPWMIO:
	stwu	r1,-0x1C(r1)	   	# Save old SP into SP-0x1C, SP=SP-0x1C
	mfspr	r0,LR	           	# r0 = LR 
	stw	r29,0x10(r1)	   
	stw	r30,0x14(r1)	
	stw	r31,0x18(r1)
	stw	r0,0x20(r1)

        li      r31,0  
        addis   r31,r31,A_MIOSBase 	# r31 = Pointer for MIOS Module (0x0030 0000)

        li      r0,0x0FFF          	# b0..b3 Input, b4..b15 Output
        sth     r0,A_MPIOSMDDR(r31)	# write DDR register

        lhz     r3,A_MPIOSMDR(r31) 	# Read PORT Value
        srawi   r3,r3,4           	# Shift 12 Place to the right (12???)
        andi.   r3,r3,0x0F       	# Keep only 4 bit into r3

	lwz	r29,0x10(r1)       	# restore register and LR
	lwz	r30,0x14(r1)       
	lwz	r31,0x18(r1)
	lwz	r0,0x20(r1)
	mtspr	LR,r0
	addi	r1,r1,0x1C
	blr	


#*****************************************************************************
# prototype:   int WritePWMIO (int Value)
#-----------------------------------------------------------------------------
# value: 8 bit value. Return the previous value as result
#-----------------------------------------------------------------------------

pwm_IOMASK  = 0x0FF0

WritePWMIO:
	stwu	r1,-0x1C(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r29,0x10(r1)	   
	stw	r30,0x14(r1)	
	stw	r31,0x18(r1)
	stw	r0,0x20(r1)

        li      r31,0  
        addis   r31,r31,A_MIOSBase # r31 = Pointer for MIOS Module (0x0030 0000)

        li      r0,0x0FFF          # b0..b3 Input, b4..b31 Output
        sth     r0,A_MPIOSMDDR(r31)# write DDR register

        lhz     r30,A_MPIOSMDR(r31)# Read PORT Value
        mr      r29,r30
        andi.   r30,r30,pwm_IOMASK # keep only b20..b27
        srawi   r30,r30,4          # Shift 4 Place to the right
        rlwinm  r3,r3,4,20,27      # SHIFT Left and keep only bit 20...27
        andi.   r29,r29,!pwm_IOMASK# keep only not (b20..b27)
        andi.   r3,r3,pwm_IOMASK
        or      r3,r3,r29
        sth     r3,A_MPIOSMDR(r31) # Write PORT Value
        andi.   r3,r30,0xFF        # Keep only 8 bit into r3, r3=previous value

	lwz	r29,0x10(r1)       # restore register and LR
	lwz	r30,0x14(r1)       
	lwz	r31,0x18(r1)
	lwz	r0,0x20(r1)
	mtspr	LR,r0
	addi	r1,r1,0x1C
	blr	

#*****************************************************************************
#MPIOInit
#	Initialisation des entrees sorties
#*****************************************************************************

MPIOInit:
	stwu	r1,-0x1C(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR    
	stw	r30,0x14(r1)	
	stw	r31,0x18(r1)
	stw	r0,0x20(r1)

        li      r31,0  
        addis   r31,r31,A_MIOSBase # r31 = Pointer for MIOS Module (0x0030 0000)
	ori	r31,r31,A_MPIOSMDDR
        
	li      r30,0x0FFF         		 # b0..b3 Input, b4..b31 Output       
	sth	r30,0x0(r31)	#write DDR register
	
	li	r31,0
	addis   r31,r31,A_MIOSBase
	ori	r31,r31,A_MPIOSMDR
	li	r30,0
	sth	r30,0x0(r31)	#Write DR register 0 en input et output

	lwz	r30,0x14(r1)       # restore register and LR
	lwz	r31,0x18(r1)
	lwz	r0,0x20(r1)
	mtspr	LR,r0
	addi	r1,r1,0x1C
	blr	