#*****************************************************************************
#* File:  INTASM.S
#* Version: 1.00,   20/10/1999
#* Author:  Philippe TECHER,  Virtual Micro Degin (ptecher@vmdesign.com) 
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - ....
#-----------------------------------------------------------------------------

        .name           "intasm.s"
	.include	"555regsdef.s"

# Exported function
        .xdef           InitInterrupt

        .text

#*****************************************************************************
# Syntax
# A_NameHi : Refers MSB 16bit Address  for "name" Module 
# A_NameLo : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
#-----------------------------------------------------------------------------

A_INTBASE        = 0xFFF00500   # Base address for EXTERNAL INTERRUPT

A_FPUBASE        = 0xFFF00800   # Base address for FPU exception

A_DECBASE	 = 0xFFF00900   # Base address for DEC exception

A_SIUBase        = 0x002F  # base to access SIU Module   
A_SIUOffset      = 0xC000  # Offset to add to obtain SIU base address

A_SIUMCR	 = 0x0000  # Section 6.13.1.1
A_SIPEND         = 0x0010  # Interrupt Pending register (6.13.2.1)
A_SIMASK         = 0x0014  # Mask (6.13.2.2)
A_SIEL           = 0x0018  # Level and edge for external IRQ lines
A_SIVEC          = 0x001C  # Vector(level) when an interrupt occur

A_PISCR          = 0x0240  # (6.13.4.8) 
A_PITC           = 0x0244  # (6.13.4.10)
A_PITR           = 0x0248

#*****************************************************************************
# prototype:   void InitInterrupt (int PITValue)
#-----------------------------------------------------------------------------
# PIT Period = (PITC+1) / 15625 ; 4Mhz Quartz, PLL set to 40MHz
#-----------------------------------------------------------------------------

InitInterrupt:
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0,0x1C(r1)        # [SP + 0x1C] = r0


        li      r31,0
	addis   r31,r31,A_SIUBase
	ori	r31,r31,A_SIUOffset

	li	r0,0x0800          # pin IRQn as SGPIO instead of IRQ !!
	stw	r0,A_SIUMCR(r31)   # WARNING IRQ0 is a NON maskable interrupt

        li      r0,0x0480          # PIT irq Disable, and Stopped, set IRQ Level to 4
        sth     r0,A_PISCR(r31)

        cmpi	CR0,0,r3,0 	   # If PITValue=0 Then disable PIT IRQ	
	beq	IntI_00
        sth     r3,A_PITC(r31)     # Store PITValue (15 => 1,024 mS)

        li      r0,0x0485          # PIT irq enable, and running, set IRQ Level to 4
        sth     r0,A_PISCR(r31)
IntI_00:
        li      r0,0x5555          # Enable only Internal INTERRUPT
        sth     r0,A_SIMASK(r31)

	mfmsr	r0
	ori	r0,r0,0xA042	   # Enable external Interrupt, FP enable, FFF00000 for IT enable
	andi.	r0,r0,0xFFFF	   # keep only the 16 Low bit
        mtmsr   r0

	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	


#*****************************************************************************
# This is the basic PIT Interrupt handler, after RESET the PS bit, it may
# call an external procedure
#-----------------------------------------------------------------------------

xInterruptHandlerPIT:

        li      r31,0
        addis   r31,r31,A_SIUBase
	ori	r31,r31,A_SIUOffset

        lhz     r0,A_PISCR(r31)    # Read PISCR
        ori     r0,r0,0x0080       # reset PS bit by writting a one (see: 6.13.4.8)
        sth     r0,A_PISCR(r31)	   # If this bit is not reset, then another interrupt
			           # will occurs just after the end of this one.

        b       InterruptHandlerPIT                          		
	


#*****************************************************************************


interr0x500:     
	stwu	r1,-0xA8(r1)	# Make a N word frame in the stack
	stw	r0,0x0C(r1)	# Save r0
        mfspr   r0,SRR0
	stw	r0,0x10(r1)	# Save SRR0
        mfspr   r0,SRR1
	stw	r0,0x14(r1)	# Save SRR1
        mfspr	r0,LR		# Get LR Link Register 
	stw	r0,0x18(r1)	# Save it
	mfcr	r0		# Get CR Condition Register
	stw	r0,0x1C(r1)	# Save it
	stw	r31,0x20(r1)	# Save it

	stw 	r2,0x28(r1)
	stw	r3,0x2C(r1)	# R3 is ALWAYS used as result for a function !
	stw 	r4,0x30(r1)
	stw 	r5,0x34(r1)
	stw 	r6,0x38(r1)
	stw 	r7,0x3C(r1)
	stw 	r8,0x40(r1)
	stw 	r9,0x44(r1)
	stw 	r10,0x48(r1)
	stw 	r11,0x4C(r1)
	stw 	r12,0x50(r1)
	stw 	r13,0x54(r1)
	stw 	r14,0x58(r1)
	stw	r15,0x5C(r1)
	stw 	r16,0x60(r1)
	stw 	r17,0x64(r1)
	stw 	r18,0x68(r1)
	stw 	r19,0x6C(r1)
	stw 	r20,0x70(r1)
	stw 	r21,0x74(r1)
	stw 	r22,0x78(r1)
	stw 	r23,0x7C(r1)
	stw 	r24,0x80(r1)
	stw 	r25,0x84(r1)
	stw 	r26,0x88(r1)
	stw 	r27,0x8C(r1)
	stw 	r28,0x90(r1)
	stw 	r29,0x94(r1)
	stw 	r30,0x98(r1)

        #----------- Interrupt handler is here
	mfspr	r0,148 		#Clear ECR
	
        li      r31,0
        addis   r31,r31,A_SIUBase
	ori	r31,r31,0xC000

        lbz     r0,A_SIVEC(r31) # r0 = Interrupt Vector 
        andi.   r0,r0,0x003C

        li      r31,InterruptTable@l
        andi.   r31,r31,0xFFFF
        addis   r31,r31,InterruptTable@h
        add     r31,r31,r0
        lwz     r0,0(r31)
 
	mtspr	LR,r0		# LR = r0 
        blrl                    # Branch to R0

	
        #----------- End of Interrupt handler

	lwz 	r2,0x28(r1)
	lwz 	r3,0x2C(r1)
	lwz 	r4,0x30(r1)
	lwz 	r5,0x34(r1)
	lwz 	r6,0x38(r1)
	lwz 	r7,0x3C(r1)
	lwz 	r8,0x40(r1)
	lwz 	r9,0x44(r1)
	lwz 	r10,0x48(r1)
	lwz 	r11,0x4C(r1)
	lwz 	r12,0x50(r1)
	lwz 	r13,0x54(r1)
	lwz 	r14,0x58(r1)
	lwz	r15,0x5C(r1)
	lwz 	r16,0x60(r1)
	lwz 	r17,0x64(r1)
	lwz 	r18,0x68(r1)
	lwz 	r19,0x6C(r1)
	lwz 	r20,0x70(r1)
	lwz 	r21,0x74(r1)
	lwz 	r22,0x78(r1)
	lwz 	r23,0x7C(r1)
	lwz 	r24,0x80(r1)
	lwz 	r25,0x84(r1)
	lwz 	r26,0x88(r1)
	lwz 	r27,0x8C(r1)
	lwz 	r28,0x90(r1)
	lwz 	r29,0x94(r1)
	lwz 	r30,0x98(r1)

	lwz	r31,0x20(r1)
	lwz	r0,0x1C(r1)	# Get CR off stack
	mtcrf	0xFF,r0		# Restore CR Condition Register
	lwz	r0,0x18(r1)	# Get LR off stack
	mtspr	LR,r0		# Restore LR Link Register 
      	lwz	r0,0x14(r1)	# Get SRR1 off stack
	mtspr	srr1,r0 	# Restore SRR1 Machine Status Save/Restore Register 1
	lwz	r0,0x10(r1)	# Get SRR0 off stack
	mtspr	srr0,r0 	# Restore SRR0 Machine Status Save/Restore Register 0
	lwz	r0,0x0C(r1)	# Restore r0
 	addi	r1,r1,0xA8      # restore Stack: r1 = r1 + n
        rfi

InterruptTable:

        .long InterruptHandlerIRQ0   # 0
        .long InterruptHandlerCAN0   # 1   Level 0x80 (0)
        .long InterruptHandlerIRQ1   # 2
        .long InterruptHandlerCAN1   # 3   Level 0x40 (1)
        .long InterruptHandlerIRQ2   # 4
        .long InterruptHandler2      # 5   Level 0x20 (2)
        .long InterruptHandlerIRQ3   # 6
        .long InterruptHandlerUART   # 7   Level 0x10 (3)
        .long InterruptHandlerIRQ4   # 8
        .long InterruptHandlerSPI    # 9   Level 0x08 (4)
        .long InterruptHandlerIRQ5   # 10
        .long xInterruptHandlerPIT   # 11  Level 0x04 (5)
        .long InterruptHandlerIRQ6   # 12
        .long InterruptHandler6      # 13  Level 0x02 (6)
        .long InterruptHandlerIRQ7   # 14
        .long InterruptHandler7      # 15  Level 0x01 (7)

#-----------------------------------------------------------------------------
# Here is the interrupt handler, it will call the specific address located
# in InterruptTable and assigned at Link moment.

#       .org A_INTBASE          # origine to 0x0500 (External Interrupt)
.section .ithandler
	b	interr0x500

#       .org A_FPUBASE          # origine to 0x0800 (FPU Interrupt)
.section .fpu
	mfmsr	r0
	ori	r0,r0,0x2002
	mtmsr	r0
	rfi


#       .org A_DECBASE          # origine to 0x0900 (DEC Exeption)
.section .dec
	li	r0,0
	mtspr	22,r0		# remise a 0 du decrementer
	rfi

