#*****************************************************************************
#* File:  LCDASM.S
#* Version: 1.00,   20/10/1999
#* Author:  Philippe TECHER,  Virtual Micro Degin (ptecher@vmdesign.com) 
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - ....
#-----------------------------------------------------------------------------

        .name           "lcdasm.s"
        .text
	.include        "555regsdef.s"

# Exported function

        .xdef          	LCDInit
        .xdef          	LCDPutChar
        .xdef          	LCDGetChar
        .xdef          	LCDPutCmd

#*****************************************************************************
# Syntax
# A_NameHi : Refers MSB 16bit Address  for "name" Module 
# A_NameLo : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
#-----------------------------------------------------------------------------

A_QADCBase       = 0x0030  # base to access QADC Module   
Ofs_ADC0         = 0x0000  # Offset for module n0
Ofs_ADC1         = 0x0400  # Offset for module n1

A_QADC64MCR      = 0x4800
A_QADC64TEST     = 0x4802
A_QADC64INT      = 0x4804
A_PORTQA         = 0x4806 
A_PORTQB         = 0x4807 
A_DDRQA          = 0x4808 
A_QACR0          = 0x480A
A_QACR1          = 0x480C
A_QACR2          = 0x480E 
A_QASR0          = 0x4810 
A_QASR1          = 0x4812 

#-----------------------------------------------------------------------------
# Value

V_DDRA_INITLCD   = 0x7F       # All PA0..PA6 as Ouput, PA7 as Input
V_DDRA_WRITELCD  = 0x7F       # All PA0..PA6 as Ouput, PA7 as Input
V_DDRA_READLCD   = 0x70       # PA0..PA3, PA7 as Intput, PA4,PA5,PA6 as Output
V_LCDDEFAULT     = 0x00       # All line to "0"

LCD_MODE4BIT     = 0x02       # D5 = 1  -> Inbterface in 4Bit Mode 
LCD_CONFIG       = 0x2C       # Mode 2 Line (0x24 for 1 line mode)
LCD_SETCURSOR    = 0x0C       # 0c Display ON, Cursor OFF, No blink
LCD_CLEAR        = 0x01       # Clear LCD
LCD_SETMODE      = 0x06       # No shift, Increment mode

MaxLCDRetry      = 8000       # Will retry to read busy flag 8000 time (eg=400us)
ErrorLCD         = -1         # Error returned when busy flag is ALWAYS set !

#*****************************************************************************
# LCDSendDATAByte : output a DATA byte to the LCD in 4 bit mode
#-----------------------------------------------------------------------------

LCDSendDATAByte:
        li      r0,0x20            # Set RS signal first
        stb     r0,A_PORTQA(r31)
        eieio
	nop
	nop

        mr      r0,r3
        srawi   r3,r3,4            # Shift 4 Place to the right
        andi.   r3,r3,0x0F         # Keep only bit b7..b4
        ori     r3,r3,0x60         # Set E and RS signal
        stb     r3,A_PORTQA(r31)
        eieio
	nop
        nop                        # Wait at least 250nS 

        andi.   r3,r3,0x2F         # Keep only 4 first bit b3..b0
        stb     r3,A_PORTQA(r31)   # Reset E Signal, keep RS set
        eieio
	nop
        nop                        # Wait at least 300nS

        mr      r3,r0

        andi.   r3,r3,0x0F         # Keep only bit b3..b0
        ori     r3,r3,0x60         # Set E and RS signal
        stb     r3,A_PORTQA(r31)
        eieio
	nop
        nop                        # Wait at least 300nS

        andi.   r3,r3,0x2F         # Keep only 4 first bit b3..b0
        stb     r3,A_PORTQA(r31)   # Reset E Signal, keep RS set
        eieio
	nop
        nop                        # Wait at least 300nS

        li      r0,0x00            # ReSet RS and E signals
        stb     r0,A_PORTQA(r31)
        eieio
	nop
        nop                        # Wait at least 300nS
        blr                        # Return


#*****************************************************************************
# LCDSendCMDByte : output a COMMAND byte to the LCD in 4 bit mode
#-----------------------------------------------------------------------------
# Send HALF byte to the LCD is used only at the begining of LCD Initialization
# R3 = Value to output
#-----------------------------------------------------------------------------

LCDSendCMDByte:
        mr      r0,r3
        srawi   r3,r3,4            # Shift 4 Place to the right
        andi.   r3,r3,0x0F         # Keep only bit b7..b4
        ori     r3,r3,0x40         # Set E signal
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
                                   # Wait at least 250nS 
        andi.   r3,r3,0x0F         # Keep only 4 first bit b3..b0
        stb     r3,A_PORTQA(r31)   # Reset E Signal
        eieio
	nop
	nop
                                   # Wait at least 300nS
        mr      r3,r0

LCDSendHalfByte:
        andi.   r3,r3,0x0F         # Keep only bit b3..b0
        ori     r3,r3,0x40         # Set E signal
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
                                   # Wait at least 250nS
        andi.   r3,r3,0x0F         # Keep only 4 first bit b3..b0
        stb     r3,A_PORTQA(r31)   # Reset E Signal
        eieio
	nop
	nop
                                   # Wait at least 300nS
        mr      r3,r0
        blr                        # Return


#*****************************************************************************
# LCDGetCMDByte : Read COMMAND byte from the LCD in 4 bit mode
#-----------------------------------------------------------------------------
# R3 = Value read from LCD
#-----------------------------------------------------------------------------

LCDGetCMDByte:
        li      r3,0x10            # Set RW Signal
        stb     r3,A_PORTQA(r31)
        eieio
        li      r3,0x50            # Set E AND RW Signals
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
        lbz     r0,A_PORTQA(r31)
        li      r3,0x10
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
        li      r3,0x50            # Set E AND RW Signals
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
        lbz     r3,A_PORTQA(r31)
        andi.   r3,r3,0x0F
        rlwinm  r0,r0,4,24,27      # SHIFT Left 4 time and keep only bit 24..27
        or      r3,r3,r0
        li      r0,0x00
        stb     r0,A_PORTQA(r31)
        eieio
	nop
	nop
        andi.   r0,r3,0x80         # Test busy flag (b7) to set/reset Z Flag
        blr                        # Return with Z Flag set if NO Busy

#*****************************************************************************
# LCDGetDataByte : Read Char byte from the LCD in 4 bit mode
#-----------------------------------------------------------------------------
# R3 = Value read from LCD
#-----------------------------------------------------------------------------

LCDGetDataByte:
        li      r3,0x30            # Set RW and RS Signals
        stb     r3,A_PORTQA(r31)
        eieio
        li      r3,0x70            # Set RW and RS and E Signals
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
        lbz     r0,A_PORTQA(r31)   # read data
        li      r3,0x30            # reset E signal
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
        li      r3,0x70            # Set E AND RW Signals
        stb     r3,A_PORTQA(r31)
        eieio
	nop
	nop
        lbz     r3,A_PORTQA(r31)
        andi.   r3,r3,0x0F
        rlwinm  r0,r0,4,24,27      # SHIFT Left 4 time and keep only bit 24..27
        or      r3,r3,r0
        li      r0,0x00
        stb     r0,A_PORTQA(r31)
        eieio
	nop
	nop
        blr                        # Return 

#*****************************************************************************
# Wait various time: 39s, 400s, 1.53ms, 13ms 
#-----------------------------------------------------------------------------

Wait13MS:
        li      r0,0x7F00
        b       WaitLoop
Wait400US:
        li      r0,1000
        b       WaitLoop
Wait1530US:
        li      r0,3900
WaitLoop:
        mtspr   9,r0
wLoop:
        nop
        nop
        nop
        eieio
        bdnz   wLoop
        blr

#*****************************************************************************
# C Function prototype:
#*****************************************************************************

#*****************************************************************************
# prototype:  int LCDInit (void)
#-----------------------------------------------------------------------------

LCDInit:
        stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r30,8(r1)          # [SP + 0x08] = r30
        stw	r31,0xC(r1)        # [SP + 0x0C] = r31
        stw	r0,0x14(r1)        # [SP + 0x14] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase # r31 = Pointer for QADC Module (0x0030 0000)

        li      r3,V_DDRA_INITLCD  # Configure LCD Port (PORTQA section A)
        stb     r3,A_DDRQA(r31)   

	li      r3,V_LCDDEFAULT    # Default state for RS, EN, RW 		
        stb     r3,A_PORTQA(r31)   
	eieio			   
        bl      Wait13MS           # Wait 13ms
        bl      Wait13MS           # Wait 13ms

        li      r3,LCD_CONFIG
        bl      LCDSendCMDByte
        bl      Wait13MS

        li      r3,LCD_CONFIG
        bl      LCDSendCMDByte
        bl      Wait400US

        li      r3,LCD_CONFIG
        bl      LCDSendCMDByte
        bl      Wait400US

        li      r3,LCD_SETCURSOR
        bl      LCDSendCMDByte
        bl      Wait400US          # Wait 400 s

        li      r3,LCD_CLEAR
        bl      LCDPutCmd
        bl      Wait13MS

        li      r3,LCD_SETMODE
        bl      LCDPutCmd          # r3 will return 0 if OK

        lwz	r30,0x8(r1)
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return


#*****************************************************************************
# prototype:   int LCDPutChar (char x)
#-----------------------------------------------------------------------------
# Return 0 if sucess to write a CHR into the LCD, else 1, busy flag is set
# and LCD seems halted (or broken).
#-----------------------------------------------------------------------------

LCDPutChar:	

	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0 ,0x1C(r1)       # [SP + 0x1C] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase # r31 = Pointer for QADC Module (0x0030 0000)

        mr      r30,r3             # Save CHR to write into r30

        li      r3,V_DDRA_READLCD  # Configure LCD Port for READ Operation
        stb     r3,A_DDRQA(r31)

        li      r0,MaxLCDRetry     # will try during about 500uS
        mtspr   9,r0               # in fact try 10000 time to read Busy flag

LCDPut_00:
        bl      LCDGetCMDByte      # Test If Busy flag = 0 
        beq     LCDPut_01          # If flag=0 ,we can output the CHR to the LCD
        bdnz    LCDPut_00          # retry again until CTR = 0

        li      r3,V_DDRA_WRITELCD # Configure LCD Port for WRITE Operation
        stb     r3,A_DDRQA(r31)
        li      r3,ErrorLCD        # Error, LCD cannot be read        
        b       LCDPut_02

LCDPut_01:
        li      r3,V_DDRA_WRITELCD # Configure LCD Port for WRITE Operation
        stb     r3,A_DDRQA(r31)
        mr      r3,r30             # OK, now write the CHR
        bl      LCDSendDATAByte
        li      r3,0

LCDPut_02:
	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:   int LCDPutCMD (char x)
#-----------------------------------------------------------------------------
# Return 0 if sucess to write a COMMAND into the LCD, else
# ErrorLCD if busy flag is set
#-----------------------------------------------------------------------------

LCDPutCmd:	
	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r31,0x14(r1)	   # [SP + 0x14] = r31
	stw	r0 ,0x1C(r1)       # [SP + 0x1C] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase # r31 = Pointer for QADC Module (0x0030 0000)

        mr      r30,r3             # Save CHR to write into r30

        li      r3,V_DDRA_READLCD  # Configure LCD Port for READ Operation
        stb     r3,A_DDRQA(r31)

        li      r0,MaxLCDRetry     # will try during about 500uS
        mtspr   9,r0               # in fact try 10000 time to read Busy flag

LCDPut_10:
        bl      LCDGetCMDByte      # Test If Busy flag = 0 
        beq     LCDPut_11          # If flag=0 ,we can output the CHR to the LCD
        bdnz    LCDPut_10          # retry again until CTR = 0

        li      r3,V_DDRA_WRITELCD # Configure LCD Port for WRITE Operation
        stb     r3,A_DDRQA(r31)
        li      r3,ErrorLCD        # Error, LCD cannot be read        
        b       LCDPut_12

LCDPut_11:
        li      r3,V_DDRA_WRITELCD # Configure LCD Port for WRITE Operation
        stb     r3,A_DDRQA(r31)
        mr      r3,r30             # OK, now write the CHR
        bl      LCDSendCMDByte

        li      r3,0

LCDPut_12:
	lwz	r30,0x10(r1)       # restore register and LR
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:   int LCDGetChar (char *x)
#-----------------------------------------------------------------------------
# Return: Current Address and read CHAR if OK and 0x8001 if busy flag is set
#-----------------------------------------------------------------------------

LCDGetChar:	
	stwu	r1,-0x1C(r1)	   # Save old SP into SP-0x1C, SP=SP-0x1C
	mfspr	r0,LR	           # r0 = LR 
	stw	r29,0x10(r1)	   # [SP + 0x10] = r29
	stw	r30,0x14(r1)	   # [SP + 0x14] = r30
	stw	r31,0x18(r1)	   # [SP + 0x18] = r31
	stw	r0 ,0x20(r1)       # [SP + 0x20] = r0

        li      r31,0  
        addis   r31,r31,A_QADCBase # r31 = Pointer for QADC Module (0x0030 0000)

        mr      r30,r3             # Save CHR address into r30

        li      r3,V_DDRA_READLCD  # Configure LCD Port for READ Operation
        stb     r3,A_DDRQA(r31)

        li      r0,MaxLCDRetry     # will try during about 500uS
        mtspr   9,r0               # in fact try 10000 time to read Busy flag

LCDGet_00:
        bl      LCDGetCMDByte      # Test If Busy flag = 0 
        beq     LCDGet_01          # If flag=0 ,we can read the CHR from the LCD
        bdnz    LCDGet_00          # retry again until CTR = 0

        # li      r3,0x8001          # Error, LCD cannot be read        
        li      r3,-0x7FFF          # Error, LCD cannot be read        
        b       LCDGet_02

LCDGet_01:                         # OK, now READ the CHR
        mr      r29,r3
        bl      LCDGetDataByte
        stb     r3,0(r30)          # Store the byte read from LCD
        mr      r3,r29

LCDGet_02:
        li      r0,V_DDRA_WRITELCD # Configure LCD Port for WRITE Operation
        stb     r0,A_DDRQA(r31)

	lwz	r29,0x10(r1)       # restore register and LR
	lwz	r30,0x14(r1)       
	lwz	r31,0x18(r1)
	lwz	r0,0x20(r1)
	mtspr	LR,r0
	addi	r1,r1,0x1C
	blr	

