#*****************************************************************************
#* File:  UARTASM.S
#* Version: 1.00,   20/10/1999
#* Author:  Philippe TECHER,  Virtual Micro Degin (ptecher@vmdesign.com) 
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - ....
#-----------------------------------------------------------------------------

        .name           "uartasm.s"
        .text
	.include        "555regsdef.s"

# Exported function

        .xdef          	UARTInit
	.xdef		UARTClose
        .xdef          	UARTGetChar
        .xdef          	UARTGetWaitChar
        .xdef           UARTGetNbChar
        .xdef          	UARTPutChar

        .xdef          	InitSPICoder
        .xdef          	CloseSPICoder
        .xdef          	SetZeroSPICoder
        .xdef          	ReadSPICoder

        .xdef         	SetLED
	.xdef		InterruptHandlerUART

#-------------- Switch to DATA Space
        .data 

UARTDataBase1:   .skip 0x10
UARTDataBase2:   .skip 0x10

UARTSizeRx       = 0x00  # Size in byte of the RX buffer (2 bytes, read as HALF-WORD)
UARTOrgRx        = 0x04
UARTPtrRx        = 0x08  # Current pointer for RX buffer (4 bytes, read as WORD)
UARTNbRx         = 0x0C  # Number of char received and present in the buffer (2 bytes, read as HALF-WORD)
UARTStatRx       = 0x10  # Copy of STATUS register of UART number 1 (to know if there is error) (2 bytes, read as HALF-WORD)


#-------------- END Switch to DATA Space
                 .text  # switch back to code segment
                 
#*****************************************************************************
# FSys is used to compute baud rate divider in InitSCC function
# it represents exact system f=ency, here 40Mhz
#-----------------------------------------------------------------------------
                        
FSys             = 40           # System f=ency 

#*****************************************************************************
# Error code for SCC function
#-----------------------------------------------------------------------------

Err_SCCFrame     = -1           # Error, the combination Parity + nBit + nStop is
                                   # not=al to 9 or 10 (10 and 11 bit frame with Start bit)
Err_SCCUart      = -2           # Error UART number must be 0 or 1
Err_SCCNoChar    = -1           # NO Char present in the buffer

#*****************************************************************************
# Syntax
# A_NameHi : Refers MSB 16bit Address  for "name" Module 
# A_NameLo : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
#-----------------------------------------------------------------------------

A_QSMCMHi	 = 0x0030   	
A_OFFSETSCC2     = 0x0018  # Offset to add for SCC2

# Global register 
A_QSMCRLo	 = 0x5000   
A_QDSCI_ILLo	 = 0x5004   
A_QSPI_ILLo	 = 0x5006

# SCI1
A_SCC1R0Lo	 = 0x5008   
A_SCC1R1Lo	 = 0x500A   
A_SC1SRLo	 = 0x500C   
A_SC1DRLo	 = 0x500E

# Pin Control Register  
A_PORTQSLo	 = 0x5014   
A_PQSPAR_DDRQSLo = 0x5016

# QSPI Registers 
A_SPCR0Lo	 = 0x5018  
A_SPCR1Lo	 = 0x501A  
A_SPCR2Lo	 = 0x501C  
A_SPCR3Lo	 = 0x501E  
A_SPSRLo	 = 0x501F

# SCI2 
A_SCC2R0Lo	 = 0x5020  
A_SCC2R1Lo	 = 0x5022  
A_SC2SRLo	 = 0x5024  
A_SC2DRLo	 = 0x5026

# QSCI 
A_QSCI1CRLo	 = 0x5028  
A_QSCI1SRLo	 = 0x502A 
A_SCTQLo	 = 0x502C  
A_SCRQLo	 = 0x504C

# RAM  Registers  
A_REC_RAMLo	 = 0x5140   
A_TRAN_RAMLo	 = 0x5180
A_COMD_RAMLo	 = 0x51C0

#-----------------------------------------------------------------------------

A_MIOSBase       = 0x0030
A_MPIOSMDR       = 0x6100 # MPIOSM Data Register
A_MPIOSMDDR      = 0x6102 # Data Direction Register (MPIOSMDDR)

#*****************************************************************************
# Default value definition 
#-----------------------------------------------------------------------------

V_QSMCR		 = 0x0080  # 14.5.5 QSM Configuration Register

                              # See 14.5.7 QSM Interrupt Vector Register (QIVR)
V_QDSCI_IL   	 = 0x0300  # Dual SCI Interrut Level Register, here level 3     
V_QSPI_IL	 = 0x0004  # QSPI Interrut Level Register ,    here level 4   
V_PORTQS         = 0x051C  # Port Qs Data Register (LED ON, SS=1) 14.6.3
V_PORTQS_LEDON	 = 0x051C  # default value for SCK is 1 !!!  
V_PORTQS_LEDOFF  = 0x050C
V_LEDMASK        = 0x0010  # Value to mask the LED

V_PQSPAR_DDRQS   = 0x037E  # Pin Asignement & Data Direction Register 
                              # All as output by default

                              # 14.7.1.1 QSPI Control Register 0
V_SPCR0 	 = 0x82C8  # MSTR=1, CPOL=1, CPHA=0, Bits=16, 100Khz 
                              # 14.7.1.2 QSPI Control Register 1 (SPCR1)
# V_SPCR1_ON       = 0x8020  # SPI enabled, delay after transmit = 25,6us    
V_SPCR1_ON       = -0x7FE0  # SPI enabled, delay after transmit = 25,6us    
V_SPCR1_OFF      = 0x0020  # SPI DISABLED, delay after transmit = 25,6us

V_SPCR2          = 0x0000  # no interrupt, pointer set to 0, ONE command to send
V_SPCR3          = 0x0000  # no loop, no interrupt# 14.7.1.5 QSPI Control Register 3


#*****************************************************************************
#  Entry: r3 = nUart number (0 or 1), 
#  Output: r31 point to QSMCM module for UART 1 or 2, Zero flag
#          r28 point to UARTDataBasex depending channel 0 or 1 is selected
#  is reset to return error in r3 register
#-----------------------------------------------------------------------------

SetUartNum:
        li      r28,UARTDataBase1@l
        andi.   r28,r28,0xFFFF
        addis   r28,r28,UARTDataBase1@h

        li      r31,0  
        addis   r31,r31,A_QSMCMHi  # r31 = Pointer for Serial Module (0x0030 0000)
        cmpi	cr0,0,r3,0
        beq     SetSCC_00
        cmpi    cr0,0,r3,1
        bne     SetSCC_01
        addi    r31,r31,A_OFFSETSCC2
        li      r28,UARTDataBase2@l
        andi.   r28,r28,0xFFFF
        addis   r28,r28,UARTDataBase2@h

        cmpi    cr0,0,r3,1         # set ZERO flag
SetSCC_00:
        blr
SetSCC_01:
        li      r3,Err_SCCUart
        blr                        # Return: ZERO all is OK, Non ZERO, nUart number is out of range

#*****************************************************************************
# prototype:   
#   int UARTInit (int UartNb, int Baud, int nbit, int nstop, int parity,
#                int ButPtr, int BufSize)
#-----------------------------------------------------------------------------
# UartNb = UART n0 or UART n1
# return: error code or 0 if all is OK
# UARTInit initialize also SGPIO pin, so any value that was previously
# written into this port is definitivly lost (LED value for example).
#-----------------------------------------------------------------------------

FUart            = (FSys * 1000000) / 32  # = 1250000
FUartLo          = (FUart & 0xffff)       # Get LoWord of FUart
FUartHi          = (FUart >> 16)          # Get HiWord of FUart

SCCx_DEFAULT     = 0x002C       # Receiver AND Transmitter ENABLED, RX Interrupt valid
SCCx_DEFAULTSTOP = 0x0000       # Receiver AND Transmitter DISABLED, Interrupt disabled

SCCx_PARITYODD   = 0x0C00
SCCx_PARITYEVEN  = 0x0400
SCCx_NOPARITY    = 0x0000
SCCx_10BIT       = 0x0000
SCCx_11BIT       = 0x0200


UARTInit:	
	stwu	r1,-0x1C(r1)	   # Save old SP into SP-0x1C, SP=SP-0x1C
	mfspr	r0,LR	           # r0 = LR 

	stw	r31,0x0C(r1)       # [SP + 0x0C] = r31
	stw	r30,0x10(r1)       # [SP + 0x10] = r30
	stw	r29,0x14(r1)       # [SP + 0x14] = r29
	stw	r28,0x18(r1)       # [SP + 0x18] = r28
	stw	r0,0x20(r1)        # [SP + 0x20] = r0

        li      r31,0  
        addis   r31,r31,A_QSMCMHi  # r31 = Pointer for Serial Module (0x0030 0000)
                                   #-----# Global register 
        li      r0,V_QSMCR	   # QSM Configuration Register
        sth     r0,A_QSMCRLo(r31)
        li      r0,V_QDSCI_IL
        sth     r0,A_QDSCI_ILLo(r31) # Dual SCI Interrut Level Register
        li      r0,V_QSPI_IL
        sth     r0,A_QSPI_ILLo(r31)# QSPI Interrut Level Register

                                   #-----# Pin Control Register   
        li      r0,V_PORTQS	   # Port Qs Data Register
        sth     r0,A_PORTQSLo(r31)
        li      r0,V_PQSPAR_DDRQS  # configure DDRQS
        sth     r0,A_PQSPAR_DDRQSLo(r31)

        bl      SetUartNum         # Obtain good r31 Offset from nUart
        bne     UARTInit_End
                                   # Compute baud rate value for SCC1R0

        stw     r8,UARTPtrRx(r28)  # Save into UARTPtrRx
        stw     r8,UARTOrgRx(r28)  # Save into UARTOrgRx
        sth     r9,UARTSizeRx(r28) # Save into UARTSize
        andi.   r0,r0,0
        stw     r0,UARTNbRx(r28)   # NbRx = 0
        stw     r0,UARTStatRx(r28) # STATUS = 0

        li      r0,0
        addis   r0,r0,FUartHi
        ori     r0,r0,FUartLo      # r0 = FUart, r4 = BaudRate
        divwu   r0,r0,r4           # SC1BR = r0 /r4 
        sth     r0,A_SCC1R0Lo(r31) # Store the good Baud rate divider

                                   # r7 = PARITY (0, 1 or 2)
        li      r30,0              # r30 = 0 if NO PARITY
        li      r0,SCCx_NOPARITY   # PARITY Choice: 0 -> No Parity
        cmpi	cr0,0,r7,0         #                1 -> Parity Odd
        beq     UARTInit_00         #                2 -> Parity Even
        li      r30,1              # r30 = 1 IF parity selected
        li      r0,SCCx_PARITYODD
        cmpi	cr0,0,r7,1
        beq     UARTInit_00
        li      r0,SCCx_PARITYEVEN
UARTInit_00:

        add     r30,r30,r6         # r30 = parity + nstop
        add     r30,r30,r5         # r30 = parity + nstop + nbit
        li      r29,SCCx_10BIT
        cmpi    cr0,0,r30,9        # is it a 10 bit frame ?
        beq     UARTInit_01
        li      r29,SCCx_11BIT
        cmpi    cr0,0,r30,10
        beq     UARTInit_01
        li      r3,Err_SCCFrame
        b       UARTInit_End
UARTInit_01:
        or      r0,r0,r29            # Add 10 or 11 bit frame type to initial word
        ori     r0,r0,SCCx_DEFAULT
        sth     r0,A_SCC1R1Lo(r31)   # Store PARITY and FRAME type into SCC1R1

        li      r0,0x0000            # no interrupt, RX & TX Queue Disabled
        sth     r0,A_QSCI1CRLo(r31)  # Store QUEUE configuration

UARTInit_End:

        lwz	r31,0x0C(r1)
        lwz	r30,0x10(r1)
        lwz	r29,0x14(r1)
        lwz	r28,0x18(r1)
        lwz	r0,0x20(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x1C         # restore Stack: r1 = r1 + 0x1C
	blr			   # return

#*****************************************************************************
# prototype:   int UARTClose (int UartNb)
#-----------------------------------------------------------------------------
# UartNb = UART n0 or UART n1, disable UART n
#    return: UART error code
#-----------------------------------------------------------------------------

UARTClose:
    
	stwu	r1,-0x14(r1)       # Save old SP into SP-0x10, SP=SP-0x10
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x0C(r1)       # [SP + 0x0C] = r31
	stw	r28,0x10(r1)       # [SP + 0x10] = r28
	stw	r0,0x18(r1)        # [SP + 0x18] = r0

        bl      SetUartNum
        bne     UClose_End

        li      r3,SCCx_DEFAULTSTOP
        sth     r3,A_SCC1R1Lo(r31)   # STOP USART n
	li	r3,0

UClose_End:

	lwz	r31,0x0C(r1)       # r31 = [SP + 0x0C]
	lwz	r28,0x10(r1)       # r31 = [SP + 0x10]
	lwz	r0,0x18(r1)        # r0  = [SP + 0x18]
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x14         # restore Stack: r1 = r1 + 0x10
	blr			   # return 

#*****************************************************************************
# prototype:   int UARTGetChar (int UartNb, char *C)
#-----------------------------------------------------------------------------
# UartNb = UART n0 or UART n1
#    return: Char read if any
#    if no char is in the buffer, then return Err_SCCNoChar
#-----------------------------------------------------------------------------

UARTGetChar:
    
	stwu	r1,-0x14(r1)       # Save old SP into SP-0x10, SP=SP-0x10
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x08(r1)       # [SP + 0x08] = r31
	stw	r29,0x0C(r1)       # [SP + 0x0C] = r29
	stw	r28,0x10(r1)       # [SP + 0x10] = r28
	stw	r0,0x18(r1)        # [SP + 0x18] = r0

        bl      SetUartNum
        bne     GetChr_End

        li      r3,Err_SCCNoChar   # Prepare R3 to reply

        lhz     r0,UARTNbRx(r28)   # r0 = UARTNbRx  (r28 point to the UART Data base)
        cmpi    cr0,0,r0,0x0000    # is r0 = 0 ?
        beq     GetChr_End
                                   # DISABLE UART SCC Interrupt while reading the CHAR
                                   #-------------------------------------
        lhz     r0,A_SCC1R1Lo(r31) # Get SCCxR1 (see: 14.8.3)
        andi.   r0,r0,0xFF0F       # NO INTERRUPT
        sth     r0,A_SCC1R1Lo(r31) # Store SCCxR1
                                   #-------------------------------------

        lhz     r3,UARTNbRx(r28)   # r3 = UARTNbRx  (r28 point to the UART Data base)
        mtspr   9,r3               # CTR = UARTNbRx
        addi    r3,r3,-1           # decrement r0
        sth     r3,UARTNbRx(r28)   # store it into UART data
        lwz     r29,UARTOrgRx(r28) # r29 = Pointer into the START buffer
        lbz     r3,0(r29)          # r3 = CHAR read

UARTGet_00:
        lbz     r0,1(r29)          # Ptr[n] = Ptr[n+1]
        stb     r0,0(r29)          
        addi    r29,r29,1          # Ptr = Ptr ++
        bdnz   UARTGet_00

        addi    r29,r29,-1         # decrement r29
        stw     r29,UARTPtrRx(r28)

        stb     r3,0(r4)           # Store the CHR read into pointer r4
        li      r0,0
        lhz     r3,UARTStatRx(r28) # r3 = RX STATUS, a char as been read       
        sth     r0,UARTStatRx(r28) # RX STATUS = 0
                                   # ENABLE UART SCC Interrupt
                                   #-------------------------------------
        lhz     r0,A_SCC1R1Lo(r31) # Get SCCxR1 (see: 14.8.3)
        ori     r0,r0,0x0020       # Valid RX INTERRUPT
        sth     r0,A_SCC1R1Lo(r31) # Store SCCxR1
                                   #-------------------------------------       
GetChr_End:

	lwz	r31,0x08(r1)       # r31 = [SP + 0x08]
	lwz	r29,0x0C(r1)       # r29 = [SP + 0x0C]
	lwz	r28,0x10(r1)       # r28 = [SP + 0x10]
	lwz	r0,0x18(r1)        # r0  = [SP + 0x14]
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x14         # restore Stack
	blr			   # return 

#*****************************************************************************
# prototype:   int UARTGetWaitChar (int UartNb, char *C)
#-----------------------------------------------------------------------------
# UartNb = UART n0 or UART n1
#    return: Char read if any
#-----------------------------------------------------------------------------

UARTGetWaitChar:
    
	stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x08(r1)       # [SP + 0x08] = r31
	stw	r28,0x0C(r1)       # [SP + 0x0C] = r28
	stw	r0,0x14(r1)        # [SP + 0x14] = r0

        bl      SetUartNum
        bne     GetWChr_End

GetWChr_Loop:
        lhz     r0,A_SC1SRLo(r31)  # Read SC1 STATUS Register 
        andi.   r0,r0,0x004F       # Keep only interesting bit (RDRF, OR, NF, FE, PF)
                                   # Rx OK: RDRF=1,  OR=0, NF=0, FE=0, PF=0
                                   # Rx wait: RDRF=0, OR=0, NF=0, FE=0, PF=0
                                   # Other condition result of error
        cmpi    cr0,0,r0,0x0040
        bne     GetWChr_00
        li      r3,0
        b       GetWChr_End

GetWChr_00:
        cmpi    cr0,0,r0,0x0000
        beq     GetWChr_Loop
        mr      r3,r0              # r3 return SC1 STATUS Register

GetWChr_End:
        lhz     r0,A_SC1DRLo(r31)  # Read a BYTE in ALL case in order to CLEAR
        stb     r0,0(r4)           # OR, NF, FE and PF if they were set, store into (&C)

	lwz	r31,0x08(r1)       # r31 = [SP + 0x08]
	lwz	r28,0x0C(r1)       # r31 = [SP + 0x0C]
	lwz	r0,0x14(r1)        # r0  = [SP + 0x14]
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return 

#*****************************************************************************
# prototype:   int UARTGetNbChar (int UartNb)
#-----------------------------------------------------------------------------
# UartNb = UART n0 or UART n1
#    return: Char read if any
#-----------------------------------------------------------------------------

UARTGetNbChar:
    
	stwu	r1,-0x14(r1)       # Save old SP into SP-0x10, SP=SP-0x10
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x0C(r1)       # [SP + 0x0C] = r31
	stw	r28,0x10(r1)       # [SP + 0x10] = r28
	stw	r0,0x18(r1)        # [SP + 0x18] = r0

        bl      SetUartNum
        bne     GetNChr_End

        lhz     r3,UARTNbRx(r28)   # r3 will return the number of char in the buffer

GetNChr_End:

	lwz	r31,0x0C(r1)       # r31 = [SP + 0x0C]
	lwz	r28,0x10(r1)       # r31 = [SP + 0x10]
	lwz	r0,0x18(r1)        # r0  = [SP + 0x18]
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x14         # restore Stack: r1 = r1 + 0x10
	blr			   # return 

#*****************************************************************************
# Here is the routine that is called each time an RX Interrupt occurs (ch0 or
# ch1). This routine can also be called on various event depending SCI setting
#-----------------------------------------------------------------------------

InterruptHandlerUART:

	stwu	r1,-0x1C(r1)	   # Save old SP into SP-0x18, SP=SP-0x1C
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x08(r1)
	stw	r30,0x0C(r1)
	stw	r29,0x10(r1)	
	stw	r28,0x14(r1)	
	stw	r0 ,0x18(r1)       

        li      r31,0  
        addis   r31,r31,A_QSMCMHi  # r31 = Pointer for Serial Module (0x0030 0000)

#---------- UART Number 1 --------------------
        lhz     r0,A_SC1SRLo(r31)  # Read SC1 STATUS Register (see: 14.8.4) 
        andi.   r0,r0,0x004F       # Keep only interesting bit (RDRF, OR, NF, FE, PF)
                                   # Rx OK: RDRF=1,  OR=0, NF=0, FE=0, PF=0
                                   # Other condition result of error
        cmpi    cr0,0,r0,0x0040    
        bne     UARTInt_01

        li      r28,UARTDataBase1@l
        andi.   r28,r28,0xFFFF
        addis   r28,r28,UARTDataBase1@h

	lhz     r29,UARTStatRx(r28)# to avoid to loose last error code, make a OR
	or	r0,r0,r29
        sth     r0,UARTStatRx(r28) # Store STATUS (SCxSR) register into UART data base
	
        lwz     r30,UARTPtrRx(r28) # r30 = Pointer to RX buffer
        lhz     r0,A_SC1DRLo(r31)  # Read a BYTE in ALL case in order to CLEAR
        stb     r0,0(r30)          # OR, NF, FE and PF if they were set, store into (&C)

        lhz     r29,UARTNbRx(r28)  # r29 = NbRx
        lhz     r0,UARTSizeRx(r28) # r0 = Buffer SIZE
        cmp     cr0,0,r0,r29
        beq     UARTInt_End

        addi    r30,r30,1          # Pointer = Pinter + 1
        stw     r30,UARTPtrRx(r28) # Save pointer into the structure
        addi    r29,r29,1          # NbRx = NbRx + 1
        sth     r29,UARTNbRx(r28)
        b       UARTInt_End

#---------- UART Number 2 --------------------
UARTInt_01:
        lhz     r0,A_SC2SRLo(r31)  # Read SC1 STATUS Register 
        andi.   r0,r0,0x004F       # Keep only interesting bit (RDRF, OR, NF, FE, PF)
                                   # Rx OK: RDRF=1,  OR=0, NF=0, FE=0, PF=0
                                   # Other condition result of error
        cmpi    cr0,0,r0,0x0040
        bne     UARTInt_End

        li      r28,UARTDataBase2@l
        andi.   r28,r28,0xFFFF
        addis   r28,r28,UARTDataBase2@h

	lhz     r29,UARTStatRx(r28)# to avoid to loose last error code, make a OR
	or	r0,r0,r29
        sth     r0,UARTStatRx(r28) # Store STATUS (SCxSR) register into UART data base
	
        lwz     r30,UARTPtrRx(r28) # r30 = Pointer to RX buffer
        lhz     r0,A_SC2DRLo(r31)  # Read a BYTE in ALL case in order to CLEAR
        stb     r0,0(r30)          # OR, NF, FE and PF if they were set, store into (&C)

        lhz     r29,UARTNbRx(r28)  # r29 = NbRx
        lhz     r0,UARTSizeRx(r28) # r0 = Buffer SIZE
        cmp     cr0,0,r0,r29
        beq     UARTInt_End

        addi    r30,r30,1          # Pointer = Pinter + 1
        stw     r30,UARTPtrRx(r28) # Save pointer into the structure
        addi    r29,r29,1          # NbRx = NbRx + 1
        sth     r29,UARTNbRx(r28)
        b       UARTInt_End

UARTInt_End:
	lwz	r31,0x08(r1)       # restore register and LR
	lwz	r30,0x0C(r1)
	lwz	r29,0x10(r1)
	lwz	r28,0x14(r1)
	lwz	r0,0x18(r1)
	mtspr	LR,r0
	addi	r1,r1,0x1C
	blr	

#*****************************************************************************
# prototype:   int UARTPutChar (int UartNb, char x)
#-----------------------------------------------------------------------------
# UartNb = UART n0 or UART n1
# return: error code or 0 if all is OK
#-----------------------------------------------------------------------------

UARTPutChar:	

	stwu	r1,-0x18(r1)	   # Save old SP into SP-0x18, SP=SP-0x18
	mfspr	r0,LR	           # r0 = LR 
	stw	r31,0x0C(r1)	   # [SP + 0x0C] = r31
	stw	r30,0x10(r1)	   # [SP + 0x10] = r30
	stw	r28,0x14(r1)	   # [SP + 0x14] = r28
	stw	r0 ,0x1C(r1)       # [SP + 0x1C] = r0

        bl      SetUartNum
        bne     PutChr_End

PutChr_Loop:
        lhz     r0,A_SC1SRLo(r31)  # Read SC1 STATUS Register 
        andi.   r0,r0,0x0100       # Keep only interesting bit (TDRE)
        cmpi    cr0,0,r0,0x0100
        bne     PutChr_Loop

	sth	r4,A_SC1DRLo(r31)  # Store half word into SC1DR, clear TC bit at the
        li      r3,0               # same time (see: 14.8.7.5)

PutChr_End:
	lwz	r31,0x0C(r1)       # restore register and LR
	lwz	r30,0x10(r1)
	lwz	r28,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	

#*****************************************************************************
# prototype:  int InitSPICoder (int Speed)
#-----------------------------------------------------------------------------
# Speed: must be as follow in order to set SPI f=ency:
#   SPI_Freq = Fsys / (2.Speed)
# - the SPI is initialized and ready to be used after that  
# - because the SPI is enabled at this time, a first transmission will occur
# WARNING: V_PORTQS will determine the default value of SCK when SPI is
# in off, so it is important to write PORTQS before
#-----------------------------------------------------------------------------

InitSPICoder:
        stwu	r1,-0x10(r1)       # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r30,8(r1)          # [SP + 0x08] = r30
        stw	r31,0xC(r1)        # [SP + 0x0C] = r31
        stw	r0,0x14(r1)        # [SP + 0x14] = r0

        li      r31,0  
        addis   r31,r31,A_QSMCMHi  # r31 = Pointer for Serial Module (0x0030 0000)

                                        #-----# Pin Control Register   
        li      r0,V_PORTQS		# Port Qs Data Register
        sth     r0,A_PORTQSLo(r31)
        li      r0,V_PQSPAR_DDRQS	# configure DDRQS
        sth     r0,A_PQSPAR_DDRQSLo(r31)

        # li      r0,0x8100          # MSTR=1, CPOL=0, CPHA=1, see: 14.7.1.1
        li      r0,-0x7F00          # MSTR=1, CPOL=0, CPHA=1, see: 14.7.1.1

        or 	r0,r0,r3           # r0 = default value OR speed
        sth     r0,A_SPCR0Lo(r31)  # SPCR0 = r0 

        li  	r0,V_SPCR2         # see: 14.7.1.3       
        sth     r0,A_SPCR2Lo(r31)  # SPCR2 = r0
        li  	r0,V_SPCR3         # see: 14.7.1.4
        sth     r0,A_SPCR3Lo(r31)  # SPCR3 = r0
        li  	r0,V_SPCR1_OFF     # see: 14.7.1.2
        sth     r0,A_SPCR1Lo(r31)  # written last because of SPI enable bit

        li	r3,0               # r3 = returned value = 0 = OK

        lwz	r30,0x8(r1)
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
	mtspr	LR,r0              # Restore Link Register (LR = r0)
	addi	r1,r1,0x10         # restore Stack: r1 = r1 + 0x10
	blr			   # return

#*****************************************************************************
# prototype:  int CloseSPICoder (void)
#-----------------------------------------------------------------------------

CloseSPICoder:
        stwu	r1,-0x10(r1)
        mfspr	r0,LR
        stw	r31,0xC(r1)
        stw	r0,0x14(r1)


        li      r31,0  
        addis   r31,r31,A_QSMCMHi  # r31 = Pointer for Serial Module (0x0030 0000)

        li  	r0,V_SPCR1_OFF     # see: 14.7.1.2
        sth     r0,A_SPCR1Lo(r31)  # disable SPI section
                                   # PORTQS will be set as "general" I/O Port
        lwz	r31,0xC(r1)
        lwz	r0,0x14(r1)
        mtspr	LR,r0
        addi	r1,r1,0x10
        blr

#*****************************************************************************
# prototype:  int SetZeroSPICoder (void)
#-----------------------------------------------------------------------------

Wait13MS:
	mfspr   r29,9
        li      r0,0x7F00
        mtspr   9,r0
wLoop:
        nop
        nop
        nop
        eieio
        bdnz   wLoop
        mtspr   9,r29
        blr

SetZeroSPICoder:
        stwu	r1,-0x18(r1)
        mfspr	r0,LR
        stw	r31,0x0C(r1)
        stw	r30,0x10(r1)
        stw	r29,0x14(r1)
        stw	r0,0x1C(r1)

        li      r31,0  
        addis   r31,r31,A_MIOSBase # r31 = Pointer for MIOS Module (0x0030 0000)

        li      r0,0x0FFF          # b0..b3 Input, b4..b31 Output
        sth     r0,A_MPIOSMDDR(r31)# write DDR register

        lhz     r30,A_MPIOSMDR(r31) # Read PORT Value
        ori     r30,r30,0x0008      # b28=1
        sth     r30,A_MPIOSMDR(r31) # set ZERO line
	eieio

        li      r0,5
        mtspr   9,r0
SetZ_00:
	bl 	Wait13MS
        bdnz    SetZ_00

        andi.   r30,r30,0xFFF7      # b28=0
        sth     r30,A_MPIOSMDR(r31) # reset ZERO line
	eieio

        li      r3,0

        lwz	r31,0x0C(r1)
        lwz	r30,0x10(r1)
        lwz	r29,0x14(r1)
        lwz	r0,0x1C(r1)
        mtspr	LR,r0
        addi	r1,r1,0x18
        blr

#*****************************************************************************
# prototype:  int ReadSPICoder (int *Value)
#-----------------------------------------------------------------------------

ReadSPICoder:
        stwu	r1,-0x14(r1)
        mfspr	r0,LR
        stw	r31,0x0C(r1)
        stw	r30,0x10(r1)
        stw	r0,0x18(r1)

        li      r31,0  
        addis   r31,r31,A_QSMCMHi  # r31 = Pointer for Serial Module (0x0030 0000)
        li      r0,0x55AA
        stw     r0,A_TRAN_RAMLo(r31)  # Store it to 1st place of TX Ram queue 
        li      r0,0x7F
        stb     r0,A_COMD_RAMLo(r31)  # Store command BYTE in CMD_QUEUE

        li  	r0,V_SPCR1_ON      # see: 14.7.1.2
        sth     r0,A_SPCR1Lo(r31)  # written last because of SPI enable bit
                                   # Now, wait until SPIF is set
RdSPI_00:
        lhz     r0,A_SPSRLo(r31)   # Read SC1 STATUS Register 
        andi.   r0,r0,0x0080       # Keep only interesting bit (SPIF)
        cmpi    cr0,0,r0,0x0080
        beq     RdSPI_00           # Loop until SPIF is set 

        lhz     r30,A_REC_RAMLo(r31) # Read SPI Value received
        srawi   r30,r30,3             # Shift 3 Place to the right

        stw     r30,0(r3)            # Store it into &Value

        lwz	r31,0x0C(r1)
        lwz	r30,0x10(r1)
        lwz	r0,0x18(r1)
        mtspr	LR,r0
        addi	r1,r1,0x14
        blr

#*****************************************************************************
# prototype:  int SetLED (int Status)
#-----------------------------------------------------------------------------
# Status = 0  -> LED OFF,    =1 -> LED ON
# return the previous state of the LED

SetLED:
        stwu	r1,-0x14(r1)     # Save old SP into SP-0x10, SP=SP-0x10
        mfspr	r0,LR
        stw	r31,0x08(r1)     # [SP + 0x08] = r30
        stw	r30,0x0C(r1)     # [SP + 0x0C] = r31
        stw	r0,0x10(r1)      # [SP + 0x10] = r0

        li      r31,0  
        addis   r31,r31,A_QSMCMHi       # r31 = Pointer for Serial Module (0x0030 0000)

                                        #-----# Pin Control Register   
        li      r0,V_PQSPAR_DDRQS	# configure DDRQS
        sth     r0,A_PQSPAR_DDRQSLo(r31)

        lhz     r30,A_PORTQSLo(r31)     

        li      r0,V_PORTQS_LEDOFF 	# Port Qs Data Register
        cmpi	cr0,0,r3,0
        beq	SetLED_00
        li      r0,V_PORTQS_LEDON  	# Port Qs Data Register
SetLED_00:
        sth     r0,A_PORTQSLo(r31)      # Update PORT QS

        andi.   r3,r30,V_LEDMASK
        beq     SetLED_01
        li	r3,1                    # r3 = previous value of the port
SetLED_01:

        lwz	r31,0x08(r1)
        lwz	r30,0x0C(r1)
        lwz	r0,0x10(r1)
	mtspr	LR,r0            # Restore Link Register (LR = r0)
	addi	r1,r1,0x14       # restore Stack: r1 = r1 + 0x14
	blr			 # return

#-----------------------------------------------------------------------------

