#*****************************************************************************
#* File: ULTRAASM.S  
#* Author:  Fabien Lydoire,  Stagiaire INRIA (Fabien.Lydoire@mail.dotcom.fr) 
#* Based on TPUASM.S written by Meftah GHRISSI,  Robosoft (meftah@robosoft.fr)
#-----------------------------------------------------------------------------
#* Modification:
#*   1.00 - Base version
#*   1.01 - Sebastien Heraud, Stagiaire INRIA (s.heraud@caramail.com)
#*   1.02 - ...
#-----------------------------------------------------------------------------
	.name		"ultraasm.s"
	.text
	.include	"555regsdef.s"


	.xdef TPUInitUltra
	.xdef UltraDisInt
	.xdef UltraEnaInt

	.xdef TPUStop
	.xdef TPUReset

	.xdef Ultra0Read
	.xdef Ultra1Read
	.xdef Ultra2Read
	.xdef Ultra3Read
	.xdef Ultra4Read
	.xdef Ultra5Read
	.xdef Ultra6Read
	.xdef Ultra7Read

	.xdef	ReadEtatCap
	.xdef MPIODis1
	.xdef MPIODis2
	.xdef MPIODis3
	.xdef MPIOEn1
	.xdef MPIOEn2
	.xdef MPIOEn3
	.xdef MPIOPWMAV
	.xdef MPIOPWMAR
			
# *************************************************
# Syntax
# A_NameHi : Refers MSB 16bit Address  for "name" Module 
# A_NameLo : Refers LSB 16bit Address  for "name" Register 
# V_Name   : Refers     16bit Value to set "name" Register 
     	A_TPU3AHi		=	0x0030

# *************************************************
# TPU Module Configuration Register
# TCR1P = 10 (divide by 8)
	A_TPUMCRLo		=	0x4000
	V_TPUMCR		=	0x7000

# *************************************************
# TPU Module Configuration Register 2
# DIV2 = 0
	A_TPUMCR2Lo		=	0x4028
	V_TPUMCR2		=	0x00A2

# *************************************************
# TPU Module Configuration Register 3
# EPSCKE = 1	Enhanced pre-scaler enable
# EPSCK	= 0xe (30)
	A_TPUMCR3Lo		=	0x402A
	V_TPUMCR3		=	0x004e

# *************************************************
# Disable Channels cleaning the two channel 
# priority bits on each of the FQD Channel Ch0 a Ch7:00
# in CPR1 Channel Priority register
# (0 0 Disable)
# (0 1 Low)
# (1 0 Medium)
# (1 1 Hight)
	A_CPR1Lo	=	0x401E		
	V_CPR1Dis	=	0x0000

# Enable servicing by asigning a Priority 
# Both channel must have the same Priority Ch0:11 a Ch7:11
	# V_CPR1Ena	=	0xFFFF
	V_CPR1Ena  	=	-0x0001

# *************************************************
# Select NITC Funtion for channel 4 5 6 7
	A_CFSR2Lo	=	0x4010
	# V_CFSR2	=	0xAAAA
	V_CFSR2		=	-0x5555

# *************************************************
# Select NITC Funtion for channel 0 1 2 3
	A_CFSR3Lo	=	0x4012
	# V_CFSR3	=	0xAAAA
	V_CFSR3		=	-0x5556

# *************************************************
# Initialise CORR_PIN_STATE_ADDR & EDGE_TIME_LSB_ADDR  
# Channel (Example)
# (0  304100 CHANNEL_CONTROL) Rising edge capture TCR1
# (0  304102 START_LINK_CHANNEL/LINK_CHANNEL_COUNT/BANK_ADDRESS) Unused
# (0  304104 MAX_COUNT) 1 capture
# (0  304108 FINAL_TRANS_TIME)
# (0  30410A LAST_TRANS_TIME)
# ********************************************************
	A_PRAM_C0_CCLo		=	0x4100
	V_PRAM_C0_CC		=	0x0007
	A_PRAM_C0_MCLo		=	0x4104
	V_PRAM_C0_MC		=	0x0000
	A_PRAM_C0_FTTLo		=	0x4108
	A_PRAM_C0_LTTLo		=	0x410A

	A_PRAM_C1_CCLo		=	0x4110
	V_PRAM_C1_CC		=	0x0007
	A_PRAM_C1_MCLo		=	0x4114
	V_PRAM_C1_MC		=	0x0000
	A_PRAM_C1_FTTLo		=	0x4118
	A_PRAM_C1_LTTLo		=	0x411A

	A_PRAM_C2_CCLo		=	0x4120
	V_PRAM_C2_CC		=	0x0007
	A_PRAM_C2_MCLo		=	0x4124
	V_PRAM_C2_MC		=	0x0000
	A_PRAM_C2_FTTLo		=	0x4128
	A_PRAM_C2_LTTLo		=	0x412A

	A_PRAM_C3_CCLo		=	0x4130
	V_PRAM_C3_CC		=	0x0007
	A_PRAM_C3_MCLo		=	0x4134
	V_PRAM_C3_MC		=	0x0000
	A_PRAM_C3_FTTLo		=	0x4138
	A_PRAM_C3_LTTLo		=	0x413A

	A_PRAM_C4_CCLo		=	0x4140
	V_PRAM_C4_CC		=	0x0007
	A_PRAM_C4_MCLo		=	0x4144
	V_PRAM_C4_MC		=	0x0000
	A_PRAM_C4_FTTLo		=	0x4148
	A_PRAM_C4_LTTLo		=	0x414A

	A_PRAM_C5_CCLo		=	0x4150
	V_PRAM_C5_CC		=	0x0007
	A_PRAM_C5_MCLo		=	0x4154
	V_PRAM_C5_MC		=	0x0000
	A_PRAM_C5_FTTLo		=	0x4158
	A_PRAM_C5_LTTLo		=	0x415A
 
	A_PRAM_C6_CCLo		=	0x4160
	V_PRAM_C6_CC		=	0x0007
	A_PRAM_C6_MCLo		=	0x4164
	V_PRAM_C6_MC		=	0x0000
	A_PRAM_C6_FTTLo		=	0x4168
	A_PRAM_C6_LTTLo		=	0x416A

	A_PRAM_C7_CCLo		=	0x4170
	V_PRAM_C7_CC		=	0x0007
	A_PRAM_C7_MCLo		=	0x4174
	V_PRAM_C7_MC		=	0x0000
	A_PRAM_C7_FTTLo		=	0x4178
	A_PRAM_C7_LTTLo		=	0x417A

# ******************************************************
# Select Single shot, no links
# in HSQR1
	A_HSQR1Lo 	=	0x4016
	V_HSQR1   	=	0x0000

# ******************************************************
# Initialize the function in
# HSRR1
	A_HSRR1Lo 	=	0x401A
	V_HSRR1   	=	0x5555

# ******************************************************
# Initialize interrupts
	A_CIERLo		=	0x400A
	A_CISRLo		=	0x4020
	A_TICRLo		=	0x4008

# ******************************************************
.macro SetReg AdRgLo,Value
		addis	r11,r0,A_TPU3AHi
		ori	r11,r11,\AdRgLo
		li	r12,\Value
		sth	r12,0(r11)
	.endm

.macro SetRegOr AdRgLo,Value
		addis	r11,r0,A_TPU3AHi
		ori	r11,r11,\AdRgLo
		lhz	r12,0(r11)
		ori	r12,r12,\Value
		sth	r12,0(r11)
	.endm

###############################
#TPUInitUltra
# Initilisation des canaux TPU
################################
TPUInitUltra:	
	# Save context r0,r11,r12,LR,CR,SRR0,SRR1
	stwu	r1,-0x30(r1)    	# Make a 3 word frame in the stack
	stw	r0,0x10(r1)      	# Save r0
	stw	r11,0x14(r1)    	# Save r11
	stw	r12,0x18(r1)     	# Save r12	
	mfspr	r0,LR       		# Get LR Link Register 
	stw	r0,0x1C(r1)     	# Save it
	li	r0,0        		# Set r0=0x0


	#configure with TPUMCRX
	SetRegOr A_TPUMCR3Lo	, V_TPUMCR3	
	SetRegOr A_TPUMCRLo	, V_TPUMCR	
	SetRegOr A_TPUMCR2Lo	, V_TPUMCR2
	
	SetReg A_CPR1Lo			,V_CPR1Dis		# Disable Channels
	SetReg A_CFSR2Lo		,V_CFSR2		# Select NITC funtion
	SetReg A_CFSR3Lo		,V_CFSR3		# Select NITC funtion

	SetReg	A_PRAM_C0_CCLo	,V_PRAM_C0_CC	# Initialize function
	SetReg	A_PRAM_C0_MCLo	,V_PRAM_C0_MC

	SetReg	A_PRAM_C1_CCLo	,V_PRAM_C1_CC	# Initialize function
	SetReg	A_PRAM_C1_MCLo	,V_PRAM_C1_MC

	SetReg	A_PRAM_C2_CCLo	,V_PRAM_C2_CC	# Initialize function
	SetReg	A_PRAM_C2_MCLo	,V_PRAM_C2_MC

	SetReg	A_PRAM_C3_CCLo	,V_PRAM_C3_CC	# Initialize function
	SetReg	A_PRAM_C3_MCLo	,V_PRAM_C3_MC
	
	SetReg	A_PRAM_C4_CCLo	,V_PRAM_C4_CC	# Initialize function
	SetReg	A_PRAM_C4_MCLo	,V_PRAM_C4_MC

	SetReg	A_PRAM_C5_CCLo	,V_PRAM_C5_CC	# Initialize function
	SetReg	A_PRAM_C5_MCLo	,V_PRAM_C5_MC

	SetReg	A_PRAM_C6_CCLo	,V_PRAM_C6_CC	# Initialize function
	SetReg	A_PRAM_C6_MCLo	,V_PRAM_C6_MC

	SetReg	A_PRAM_C7_CCLo	,V_PRAM_C7_CC	# Initialize function
	SetReg	A_PRAM_C7_MCLo	,V_PRAM_C7_MC


	SetReg A_HSQR1Lo	,V_HSQR1	# Select Single shot, no links
	SetReg A_HSRR1Lo	,V_HSRR1	# Initialize the function 
	

	addis	r11,r0,A_TPU3AHi	    	# Disable Interrupt on channel 0 1 2 3 4 5 6 7
	ori	r11,r11,A_CIERLo
	lhz	r12,0(r11)
	andi.	r12,r12,0xff00
	sth	r12,0(r11)
	
	addis	r11,r0,A_TPU3AHi    		# Read CISR
	ori	r11,r11,A_CISRLo 
	lhz	r12,0(r11)

	sth	r0,0(r11)       		# Clear interrupt status register
	
	addis	r11,r0,A_TPU3AHi    		# Select IRQ1 for interrupt
	ori	r11,r11,A_TICRLo
	li	r12,0x0200
	sth	r12,0(r11)		

	addis	r11,r0,A_TPU3AHi    		# Enable interrupts on channel 0 1 2 3 4 5 6 7
	ori	r11,r11,A_CIERLo
	lhz	r12,0(r11)
	ori	r12,r12,0xff
	sth	r12,0(r11)
 
	SetReg A_CPR1Lo	,V_CPR1Ena	        # Enable servicing by asigning a Priority 

# Restore Context r0,r11,r12,LR,CR,SRR0,SRR1
	lwz	r0,0x1C(r1)	    	# Get LR off stack
	mtspr	LR,r0    			# Restore LR Link Register 
	lwz	r12,0x18(r1)		# Restore r12
	lwz	r11,0x14(r1)		# Restore r11	
	lwz	r0,0x10(r1) 		# Restore r0
	addi	r1,r1,0x30  		# Restore Stack Pointer
	blr         				# Branch to Link Register, Return from Subroutine RTS 

###############################
#TPUStop
#  Arret des canaux TPU
################################
TPUStop:
	# Save context r0,r11,r12,LR,CR,SRR0,SRR1
	stwu	r1,-0x30(r1)    	# Make a 3 word frame in the stack
	stw	r0,0x10(r1)	        # Save r0
	stw	r11,0x14(r1)	    # Save r11
	stw	r12,0x18(r1)	    # Save r12	
	mfspr	r0,LR		        # Get LR Link Register 
	stw	r0,0x1C(r1)	        # Save it
	li	r0,0		        # Set r0=0x0

	addis	r11,r0,A_TPU3AHi	# Stop TPU3A
	ori	r11,r11,A_TPUMCRLo
	lhz	r12,0(r11)
	ori	r12,r12,0x8000
	sth	r12,0(r11)

	# Restore Context r0,r11,r12,LR,CR,SRR0,SRR1
	lwz	r0,0x1C(r1) 	    # Get LR off stack
	mtspr	LR,r0   		    # Restore LR Link Register 
	lwz	r12,0x18(r1)    	# Restore r12
	lwz	r11,0x14(r1)    	# Restore r11	
	lwz	r0,0x10(r1)  	    # Restore r0
	addi	r1,r1,0x30  	    # Restore Stack Pointer
	blr     			        # Branch to Link Register, Return from Subroutine RTS 
	
###############################
#TPUReset
# Re;ise a zero des canaux TPU
################################
TPUReset:
	# Save context r0,r11,r12,LR,CR,SRR0,SRR1
	stwu	r1,-0x30(r1)    	# Make a 3 word frame in the stack
	stw	r0,0x10(r1)	        # Save r0
	stw	r11,0x14(r1)    	# Save r11
	stw	r12,0x18(r1)    	# Save r12	
	mfspr	r0,LR		        # Get LR Link Register 
	stw	r0,0x1C(r1)	        # Save it
	li	r0,0		        # Set r0=0x0

	addis	r11,r0,A_TPU3AHi	# Reset TPU3A
	ori	r11,r11,A_TPUMCR2Lo
	lhz	r12,0(r11)
	ori	r12,r12,0x80
	sth	r12,0(r11)
	andi.	r12,r12,0xf7f
	sth	r12,0(r11)
	
	# Restore Context r0,r11,r12,LR,CR,SRR0,SRR1
	lwz	r0,0x1C(r1)     	# Get LR off stack
	mtspr	LR,r0       		# Restore LR Link Register 
	lwz	r12,0x18(r1)	    # Restore r12
	lwz	r11,0x14(r1)    	# Restore r11	
	lwz	r0,0x10(r1)     	# Restore r0
	addi	r1,r1,0x30      	# Restore Stack Pointer
	blr			            # Branch to Link Register, Return from Subroutine RTS 

###################################
#UltraDistInt
# Masque les interruptions liees aux ultra son
###################################
UltraDisInt:
	# Save context r0,r11,r12,LR
	stwu	r1,-0x30(r1)    	# Make a 3 word frame in the stack
	stw	r0,0x10(r1)     	# Save r0
	stw	r11,0x14(r1)    	# Save r11
	stw	r12,0x18(r1)    	# Save r12	
	mfspr	r0,LR   	    	# Get LR Link Register 
	stw	r0,0x1C(r1)     	# Save it
	li	r0,0        		# Set r0=0x0

	addis	r11,r0,A_TPU3AHi	# Disable Interrupt on channel 0 1 2 3 4 5 6 7
	ori	r11,r11,A_CIERLo
	lhz	r12,0(r11)
	andi.	r12,r12,0xff00
	sth	r12,0(r11)
	
	# Restore Context r0,r11,r12,LR
	lwz	r0,0x1C(r1)     	# Get LR off stack
	mtspr	LR,r0       		# Restore LR Link Register 
	lwz	r12,0x18(r1)    	# Restore r12
	lwz	r11,0x14(r1)    	# Restore r11	
	lwz	r0,0x10(r1)     	# Restore r0
	addi	r1,r1,0x30      	# Restore Stack Pointer
	blr             			# Branch to Link Register, Return from Subroutine RTS 



#*****************************************
# UltraRead (int *value)
# Read TPU channel 2 
# r3 = returned adress
#******************************************
Ultra0Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C0_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr

Ultra1Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C1_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr



Ultra2Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C2_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr

Ultra3Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C3_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr

Ultra4Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C4_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr
 
Ultra5Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C5_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr

Ultra6Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C6_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr

Ultra7Read:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,A_TPU3AHi	
	ori	r30,r30,A_PRAM_C7_FTTLo	    # r30 = A_FINAL_TRANS_TIME

	lhz	r0,0x0(r30)
	extsh	r30,r0
	stw	r0,0x0(r3)

	# Restore Context r0,r30,LR
	li 	r3,0                		# Return 0
	lwz	r30,0x10(r1)            	# Restore context
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr


#*****************************************
# ReadEtatCap (int *value)
# lit l,etat du bus de donne D0 a D15
#******************************************
ReadEtatCap:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r0,0x1C(r1)
	
	li	r30,0
	addis	r30,r30,0x0FF0	
	ori	r30,r30,0x0000	#r30=0x0FF00000
	
	lwz	r0,0x0(r30)		#lecture a vide
	extsh	r30,r0
	stw	r0,0x0(r3)		# *r30 <- r0

	li 	r3,0			# return 0

	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr


#########################################
#MPIODis1
# Met a 0 (desactive) l'I/O du groupe 1 (MPIO 9)
#########################################
MPIODis1:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	andi.	r31,r31,0xFDFF
	sth	r31,0x0(r30)
		
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr
	
#########################################
#MPIODis2
# Met a 0 (desactive) l'I/O du groupe 2 (MPIO 6)
#########################################

MPIODis2:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	andi.	r31,r31,0xFFBF
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	


#########################################
#MPIODis3
# Met a 0 (desactive) l'I/O du groupe 3 (MPIO 7)
#########################################

MPIODis3:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	andi.	r31,r31,0xFF7F
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr
	
#########################################
#MPIOEn3
# Met a 1 (active) l'I/O du groupe 3 (MPIO 7)
#########################################

MPIOEn3:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	ori	r31,r31,0x80
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr
	
	
#########################################
#MPIOEn2
# Met a 1 (active) l'I/O du groupe 2 (MPIO 6)
#########################################

MPIOEn2:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	ori	r31,r31,0x40
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	
	
	
#########################################
#MPIOEn1
# Met a 1 (active) l'I/O du groupe 1 (MPIO 9)
#########################################

MPIOEn1:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	ori	r31,r31,0x200
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr	
	
######################################
#MPIOPWAR
# Active l,alimentation des capteur ultra son arriere
######################################
MPIOPWMAR:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	ori	r31,r31,0x400
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18

	blr	

######################################
#MPIOPWAV
# Active l,alimentation des capteur ultra son avant
######################################

MPIOPWMAV:
	# Save context r0,r30,LR
	stwu	r1,-0x18(r1)
	mfspr	r0,LR
	stw	r30,0x10(r1)
	stw	r31,0x14(r1)
	stw	r0,0x1C(r1)
	
	li	r31,0
	addis	r30,r31,0x0030
	ori	r30,r30,0x6100
	lhz	r31,0x0(r30)
	andi.	r31,r31,0xFBFF
	sth	r31,0x0(r30)
	
	
	li 	r3,0		# return 0
	
	# Restore Context r0,r30,LR
	lwz	r30,0x10(r1)
	lwz	r31,0x14(r1)
	lwz	r0,0x1C(r1)
	mtspr	LR,r0
	addi	r1,r1,0x18
	blr
